DockingStation = setmetatable({ }, { __index = _G })
setfenv(1, DockingStation)

addon = DockingStation
title = "DockingStation"
configName = title .. "_Config"
settings = false																						-- To avoid issues with a global named 'settings' (it's happened)

local gsub, pairs, _pcall, tonumber, tostring, tremove, type, unpack = gsub, pairs, pcall, tonumber, tostring, tremove, type, unpack

--[[-----------------------------------------------------------------------------
Helper functions
-------------------------------------------------------------------------------]]
local ConvertSettings
do
	local function convert_name(data)
		local thereCanBeOnlyOne
		for _, sectionType in pairs(section_types) do
			for index, name in ipairs(data[sectionType]) do
				if name == title or name == configName then
					if thereCanBeOnlyOne then
						tremove(data[sectionType], index)
					elseif name == title then
						data[sectionType][index] = configName
					end
					thereCanBeOnlyOne = true
				end
			end
		end
	end

	local function convert_setting(data, oldField, newField, invert, keep)
		if type(data[oldField]) == 'nil' then return end
		if type(data[newField]) == 'nil' then
			if invert then
				data[newField] = not data[oldField]
			else
				data[newField] = data[oldField]
			end
		end
		if keep then return end
		data[oldField] = nil
	end

	local function convert_template(data)
		if type(data.template) == 'nil' then return end
		if not _pcall(format, data.template) then
			data.template = nil
		end
	end

	local function convert_tooltip(data)
		convert_setting(data, "tooltipHide", "tooltipHideCombat", false, true)
		convert_setting(data, "tooltipHide", "tooltipHideNoCombat")
		convert_setting(data, "tooltipRestrict", "tooltipHideCombat")
		if type(data.tooltipNoScale) ~= 'nil' then
			data.tooltipNoScale, data.tooltipScale = nil, nil
		end
	end

	function ConvertSettings()
		for profile, settings in pairs(addon.db.profiles) do
			convert_template(settings)
			convert_tooltip(settings)
			for name, data in pairs(settings.plugins) do
				convert_tooltip(data)
			end
			for id, data in ipairs(settings.panels) do
				convert_name(data)
				convert_template(data)
				convert_tooltip(data)
			end
			settings.plugins[configName] = settings.plugins[configName] or settings.plugins[title]
			settings.plugins[title] = nil
		end
	end
end

local function UpdateProfile()
	AllPanels("Recycle")
	settings = GetSettings()
	CreatePanel(1)
	for i = 2, #settings.panels do
		CreatePanel(i)
	end
	AllPanels("Refresh")
	PanelList:Refresh()
	PluginList:Refresh()
end

local function UpdateSharedMedia(event, type)
	if not (type == 'border' or type == 'font' or type == 'statusbar' or type == 'overlay') then return end
	UpdatePanelConfig()
	UpdatePluginConfig()
	AllPanels("Refresh")
end

RESOLUTION_HEIGHT, RESOLUTION_WIDTH, SCREEN_HEIGHT, SCREEN_WIDTH = 768, 1024, 768, 1024
SCREEN_HEIGHT_MIDDLE, SCREEN_WIDTH_MIDDLE = SCREEN_HEIGHT / 2, SCREEN_WIDTH / 2
local function UpdateResolution(event, screenWidth, screenHeight, resolutionWidth, resolutionHeight)
	RESOLUTION_HEIGHT, RESOLUTION_WIDTH, SCREEN_HEIGHT, SCREEN_WIDTH = resolutionHeight, resolutionWidth, screenHeight, screenWidth
	SCREEN_HEIGHT_MIDDLE, SCREEN_WIDTH_MIDDLE = screenHeight / 2, screenWidth / 2
	UpdatePanelConfig()
	AllPanels("Refresh")
end

--[[-----------------------------------------------------------------------------
Global to addon
-------------------------------------------------------------------------------]]
DEFAULT_FONT, LDB, LJ, LSM = [[Fonts\FRIZQT__.TTF]], LibStub('LibDataBroker-1.1'), LibStub('LibJostle-3.0'), LibStub('LibSharedMedia-3.0')
section_types = { Center = "Center", Left = "Left", Right = "Right" }
strata_layers = { 'BACKGROUND', 'LOW', 'MEDIUM', 'HIGH', 'DIALOG', 'FULLSCREEN', 'FULLSCREEN_DIALOG' }
string_categories = { "label", "text", "suffix" }

utilityFrame = CreateFrame('Button', nil, UIParent)										-- Used to work around limitations
utilityFrame:Hide()
MT_FRAME = { __index = getmetatable(utilityFrame).__index }

function pcall(...)
	local ok, retVal = _pcall(...)
	if not ok and settings.report then
		local path, line, message = retVal:match([[^.-\.-\(.-):(%d+): (.+)$]])
		if path and line then
			message = path .. "[" .. line .. "]: " .. message
		else
			message = retVal
		end
		path, line = debugstack(2):match([[^.-\.-\(.-):(%d+):]])
		if path and line then
			message = path .. "[" .. line .. "]: " .. message
		end
		geterrorhandler()(message)
	end
	return ok, retVal
end

function tremove_byVal(table, value)
	for index, indexValue in ipairs(table) do
		if indexValue == value then
			tremove(table, index)
			break
		end
	end
end

function AllPanels(method, ...)
	for id = #panels, 1, -1 do																		-- Reversed order so that panel:Recycle() will work properly
		panels[id][method](panels[id], ...)
	end
end

function ConnectAsTooltip(frame, parent)
	local _, y = parent:GetCenter()
	if y >= SCREEN_HEIGHT_MIDDLE then
		pcall(frame.SetPoint, frame, 'TOP', parent, 'BOTTOM', 0, 0)
	else
		pcall(frame.SetPoint, frame, 'BOTTOM', parent, 'TOP', 0, 0)
	end
end

function DoNothing()
end

function GetPluginLocation(name)
	local index, panel, section
	for id = 1, #panels do
		panel = panels[id]
		section, index = panel:HasPlugin(name)
		if section then
			return panel, section, index
		end
	end
end

function IsFrame(object)
	local ok, objectType = pcall(utilityFrame.GetObjectType, object)
	return ok and objectType
end

function RemoveColorCodes(string)
	string = gsub(gsub(tostring(string or ""), "\|[Rr]", ""), "\|[Cc]%x%x%x%x%x%x%x%x", "")
	return string
end

function SetTexCoord(frame, texCoord, zoom)
	local ULx, ULy, LLx, LLy, URx, URy, LRx, LRy
	zoom = zoom or 0
	if type(texCoord) == 'table' then
		ULx, ULy, LLx, LLy, URx, URy, LRx, LRy = unpack(texCoord)
		ULx, ULy, LLx, LLy, URx, URy, LRx, LRy = tonumber(ULx) or 0, tonumber(ULy) or 0, tonumber(LLx) or 0, tonumber(LLy) or 1, tonumber(URx) or 1, tonumber(URy) or 0, tonumber(LRx) or 1, tonumber(LRy) or 1
		if #texCoord == 4 then
			ULy, LLx, URx, URy, LRx, LRy = LLx, ULx, ULy, LLx, ULy, LLy
		else
			zoom = 0																						-- Need to do the math for an 8 parameter iconCoord zoom
		end
	else
	   ULx, ULy, LLx, LLy, URx, URy, LRx, LRy = 0, 0, 0, 1, 1, 0, 1, 1
	end
	pcall(frame.SetTexCoord, frame, ULx + zoom, ULy + zoom, LLx + zoom, LLy - zoom, URx - zoom, URy + zoom, LRx - zoom, LRy - zoom)
end

function SetTexture(frame, texture)
	if type(texture) == 'string' then
		pcall(frame.SetTexture, frame, texture)
	else
		frame:SetTexture([[Interface\BUTTONS\WHITE8X8]])
		if type(texture) == 'table' then
			pcall(frame.SetVertexColor, frame, unpack(texture))
		end
	end
end

--[[-----------------------------------------------------------------------------
Global
-------------------------------------------------------------------------------]]
function addon:GetNumPanels()
	return #panels
end

function addon:GetPanel(id)
	id = tonumber(id)
	return id and panels[id]
end

--[[-----------------------------------------------------------------------------
Initialize
-------------------------------------------------------------------------------]]
LibStub('LibDisplayAssist-1.2'):Register(addon, function(...)							-- Runs on first frame update to ensure proper sizing
	LibStub('LibDisplayAssist-1.2'):Unregister(addon)
	addon.db = LibStub('AceDB-3.0'):New(title .. "Settings", nil, 'Default')
	UpdateResolution(...)																			-- Refreshes panels
	ConvertSettings()
	UpdateProfile()
	for name, dataObject in LDB:DataObjectIterator() do									-- Check for plugins
		ValidateDataObject(nil, name, dataObject)
	end
	addon.db.RegisterCallback(addon, 'OnProfileChanged', UpdateProfile)
	addon.db.RegisterCallback(addon, 'OnProfileCopied', UpdateProfile)
	addon.db.RegisterCallback(addon, 'OnProfileReset', UpdateProfile)
	LDB.RegisterCallback(addon, 'LibDataBroker_DataObjectCreated', ValidateDataObject)
	LSM.RegisterCallback(addon, 'LibSharedMedia_Registered', UpdateSharedMedia)
	LSM.RegisterCallback(addon, 'LibSharedMedia_SetGlobal', UpdateSharedMedia)
	LibStub('LibDisplayAssist-1.2'):Register(addon, UpdateResolution)
end)

LSM:Register('overlay', "Line, Gradient", [[Interface\AddOns\]] .. title .. [[\Media\Line, Gradient]])
LSM:Register('overlay', "Line, Gradient (Center)", [[Interface\AddOns\]] .. title .. [[\Media\Line, Gradient (Center)]])
LSM:Register('overlay', "Line, Solid", [[Interface\AddOns\]] .. title .. [[\Media\Line, Solid]])
LSM:Register('overlay', "Gloss", [[Interface\AddOns\]] .. title .. [[\Media\Gloss]])
LSM:Register('overlay', "None", [[Interface\AddOns\]] .. title .. [[\Media\None]])

LSM:Register('statusbar', "Blizzard Gradient", [[Interface\WorldStateFrame\WORLDSTATEFINALSCORE-HIGHLIGHT]])
LSM:Register('statusbar', "Empty", [[Interface\AddOns\]] .. title .. [[\Media\None]])
LSM:Register('statusbar', "Solid", [[Interface\BUTTONS\WHITE8X8]])
