--[[-----------------------------------------------------------------------------
This is a beta version.  It is only included with DockingStation for testing.
-------------------------------------------------------------------------------]]
local LIB, REVISION = "LibOptionsAssist-0.0", 1
if not LibStub then error(LIB .. " requires LibStub", 0) end

local lib, oldRevision = LibStub:NewLibrary(LIB, REVISION)
if not lib then return end

oldRevision = oldRevision or -1

local pairs, rawset, select, type, unpack = pairs, rawset, select, type, unpack

--[[-----------------------------------------------------------------------------

Attributes:

	info		(string)	Set the information text to display.  This text is right-
							aligned and opposite the title.  The text is smaller than
							normal and slightly faded.  Only shown if the title is
							shown.  Example uses: "Author: Vrul" or "Version: 1.0"

	title		(string)	Set the title text to display.  This text is left-aligned
							in the upper-left corner.  The text is larger than normal.

-------------------------------------------------------------------------------]]
lib.Attributes = lib.Attributes or { info = { type = 'string' }, title = { type = 'string' } }
lib.Scripts = lib.Scripts or { OnClose = { }, OnEnter = { real = true }, OnHide = { }, OnLeave = { real = true }, OnOpen = { }, OnShow = { } }
lib.UserData = lib.UserData or { appName = { }, path = { } }

lib.no_op = lib.no_op or function() end
lib.get_no_op = lib.get_no_op or function() return lib.no_op end

--[[-----------------------------------------------------------------------------
All entries in the Blizzard options panel created will share the same frames and
font strings.
-------------------------------------------------------------------------------]]
if not lib.frame then
	local frame = CreateFrame('Frame', nil, UIParent)
	frame:SetScript('OnEvent', function(self)
		lib.ACD = lib.ACD or LibStub('AceConfigDialog-3.0', true)
		lib.ACR = lib.ACR or LibStub('AceConfigRegistry-3.0', true)
		if not (lib.ACD and lib.ACR and LibStub('AceGUI-3.0', true)) then return end
		self:UnregisterEvent('ADDON_LOADED')
		self:SetScript('OnEvent', nil)
		lib.widget = {
			content = lib.content,
			frame = lib.frame,
			type = LIB
		}
		LibStub('AceGUI-3.0'):RegisterAsContainer(lib.widget)
		if not lib.showing then return end
		lib.showing:Refresh()
	end)
	frame:RegisterEvent('ADDON_LOADED')
	frame:Hide()
	lib.frame = frame
end

lib.adjust_content = lib.adjust_content or function()
	local content = lib.content
	content:ClearAllPoints()
	if lib.title_bar:IsShown() then
		content:SetPoint('TOPLEFT', lib.title_bar, 'BOTTOMLEFT', 0, -8)
	else
		content:SetPoint('TOPLEFT', lib.title_bar, 'TOPLEFT', 0, 0)
	end
	content:SetPoint('BOTTOMRIGHT', lib.frame, 'BOTTOMRIGHT', -10, 10)
end

if not lib.title_bar then
	local title_bar = CreateFrame('Frame', nil, lib.frame)
	title_bar:SetPoint('TOPLEFT', lib.frame, 'TOPLEFT', 15, -15)
	title_bar:SetPoint('RIGHT', lib.frame, 'RIGHT', -10, 0)
	title_bar:SetScript('OnHide', lib.adjust_content)
	title_bar:SetScript('OnShow', lib.adjust_content)
	lib.title_bar = title_bar
end

if not lib.title then
	local title = lib.title_bar:CreateFontString(nil, 'OVERLAY', 'GameFontNormalLarge')
	title:SetJustifyH('LEFT')
	title:SetJustifyV('TOP')
	title:SetPoint('TOP', lib.title_bar, 'TOP')
	title:SetText(" ")
	title:SetHeight(title:GetStringHeight())
	lib.title_bar:SetPoint('BOTTOM', lib.frame, 'TOP', 0, -15 - title:GetStringHeight())
	title:SetAllPoints(lib.title_bar)
	title.SetValue = function(self, value)
		if value then
			self:SetText(value)
			lib.title_bar:Show()
		else
			lib.title_bar:Hide()
		end
	end
	lib.title = title
end

if not lib.info then
	local info = lib.title_bar:CreateFontString(nil, 'ARTWORK', 'GameFontNormalSmall')
	info:SetAllPoints(lib.title_bar)
	info:SetAlpha(0.75)
	info:SetJustifyH('RIGHT')
	info:SetJustifyV('MIDDLE')
	info:SetText(" ")
	info:SetHeight(info:GetStringHeight())
	info.SetValue = function(self, value)
		self:SetText(value or "")
	end
	lib.info = info
end

lib.content = lib.content or CreateFrame('Frame', nil, lib.frame)
lib.adjust_content()

--[[-----------------------------------------------------------------------------
Instead of creating a new frame and/or GUI object for each Blizzard options panel
entry made, a small table will be created and it will emulate the needed methods.
-------------------------------------------------------------------------------]]
if not lib.emulate_mt then
	local reserved = getmetatable(UIParent).__index

	lib.emulate_mt = setmetatable({
		__call = function(self)
			InterfaceOptionsFrame_OpenToCategory(self)
		end,

		__index = {
			cancel = lib.no_op,
			defaults = lib.no_op,
			okay = lib.no_op,
			refresh = lib.no_op,

			['AssignOptions'] = function(self, appName, ...)
				lib.UserData.appName[self] = appName
				if appName ~= nil and select('#', ...) > 0 then
					local path = { }
					for index = 1, select('#', ...) do
						path[#path + 1] = select(index, ...)
					end
					lib.UserData.path[self] = path
				else
					lib.UserData.path[self] = nil
				end
				self:Refresh()
			end,

			['ClearAllPoints'] = function(self)													-- Required
				if lib.showing and lib.showing ~= self then return end
				lib.frame:ClearAllPoints()
			end,

			['GetAttribute'] = function(self, attribute)
				return lib.Attributes[attribute] and lib.Attributes[attribute][self]
			end,

			['GetScript'] = function(self, script)
				return lib.Scripts[script] and lib.Scripts[script][self]
			end,

			['HasScript'] = function(self, script)
				return lib.Scripts[script] and true
			end,

			['Hide'] = function(self)																-- Required
				if lib.showing ~= self or not lib.frame:IsShown() then return end
				lib.showing = nil
				lib.frame:Hide()
				if lib.widget and lib.UserData.appName[self] then
					lib.ACD:Close(lib.UserData.appName[self])
				end
				local script = lib.Scripts.OnHide[self]
				if script then script(self) end
				script = lib.Scripts.OnClose[self]
				if script then script(self) end
			end,

			['IsShown'] = function(self)
				return lib.showing == self
			end,

			['IsVisible'] = function(self)
				return lib.showing == self and lib.frame:IsVisible()
			end,

			['Refresh'] = function(self)
				if lib.showing ~= self then return end
				for attribute, data in pairs(lib.Attributes) do
					lib[attribute]:SetValue(data[self])
				end
				if not lib.widget then return end
				local appName, path = lib.UserData.appName[self], lib.UserData.path[self]
				if not (appName and lib.ACR:GetOptionsTable(appName)) then return end
				lib.ACD:Open(appName, lib.widget, path and unpack(path))
			end,

			['SetAllPoints'] = function(self, ...)												-- Need this?
				if lib.showing and lib.showing ~= self then return end
				lib.frame:SetAllPoints(...)
			end,

			['SetAttribute'] = function(self, attribute, value)
				attribute = lib.Attributes[attribute]
				if not attribute then return end
				local type = type(value)
				if type ~= attribute.type and type ~= 'nil' then
					error(LIB .. ": <object>:SetAttribute(attribute, value) - 'value' expected " .. attribute.type .. " or nil, got " .. type, 2)
				end
				if type == 'string' then
					value = strtrim(value)
					value = value ~= "" and value or nil
				end
				attribute[self] = value
				self:Refresh()
			end,

			['SetParent'] = function(self, ...)													-- Required
				if lib.showing and lib.showing ~= self then return end
				lib.frame:SetParent(...)
			end,

			['SetPoint'] = function(self, ...)													-- Required
				if lib.showing and lib.showing ~= self then return end
				lib.frame:SetPoint(...)
			end,

			['SetScript'] = function(self, script, func)
				if not lib.Scripts[script] then
					error(LIB .. ": <object>:SetScript(script, func) - '" .. tostring(script) .. "' is not a valid script for this object", 2)
				elseif type(func) ~= 'function' and func ~= nil then
					error(LIB .. ": <object>:SetScript(script, func) - 'func' expected function or nil, got " .. type(func), 2)
				end
				lib.Scripts[script][self] = func
				if not lib.Scripts[script].real or lib.frame:GetScript(script) then return end
				lib.frame:SetScript(script, function(self, ...)
					self = lib.showing
					if not (self and lib.Scripts[script][self]) then return end
					lib.Scripts[script][self](self, ...)
				end)
			end,

			['Show'] = function(self)																-- Required
				if lib.showing then
					if lib.showing == self then return end
					lib.showing:Hide()
				end
				if self.addon then																	-- Do this before showing to avoid potential LoD issues
					lib:LoadModule(self.addon)
					self.addon = nil
				end
				lib.showing = self
				lib.frame:Show()
				self:Refresh()
				local script = lib.Scripts.OnShow[self]
				if script then script(self) end
				script = lib.Scripts.OnOpen[self]
				if script then script(self) end
			end
		},

		__newindex = function(self, key, value)
			if reserved[key] then return end
			rawset(self, key, value)
		end
	}, { __index = lib.get_no_op })

	InterfaceOptionsFrame:HookScript('OnHide', function()
		local script = lib.Scripts.OnClose[lib.showing]
		if script then script(lib.showing) end
	end)

	InterfaceOptionsFrame:HookScript('OnShow', function()
		local script = lib.Scripts.OnOpen[lib.showing]
		if script then script(lib.showing) end
	end)
end

--[=[----------------------------------------------------------------------------

LIB:AddEntry(name [, parent [, addon]])

	Add an entry to Blizzard's interface options panel under the AddOns tab.  The
	title is automatically set to name but can be changed with
	entry:SetAttribute('title', <title>).

Input:

	name		(string) 	The text to appear in the Blizzard options panel under
								the AddOns tab.

	parent	(string)		If this is to be a child entry then this should be the
								name that was used for the parent entry.

	addon		(string)		The name of an addon to be loaded (if it isn't already)
								when the entry is first selected.

Returns:

	entry		(table)		A table with the required fields to work with the
								Blizzard options panel.  This is not a frame but rather a
								container for AceGUI-3.0 widgets.  To have an options
								table fed into it use entry:AssignOptions(appName [, ...]
								), where '...' may be zero or more table keys that lead
								to a sub-table if needed.

------------------------------------------------------------------------------]=]
lib.AddEntry = lib.AddEntry or function(self, name, parent, addon)
	if type(name) ~= 'string' then
		error(LIB .. ":AddEntry(name, parent, addon) - 'name' expected string, got " .. type(name), 2)
	elseif type(parent) ~= 'string' and parent ~= nil then
		error(LIB .. ":AddEntry(name, parent, addon) - 'parent' expected string or nil, got " .. type(parent), 2)
	elseif parent and not self:HasEntry(parent) then
		error(LIB .. ":AddEntry(name, parent, addon) - 'parent' (" .. parent .. ") must exist before adding a child", 2)
	elseif type(addon) ~= 'string' and addon ~= nil then
		error(LIB .. ":AddEntry(name, parent, addon) - 'addon' expected string or nil, got " .. type(addon), 2)
	elseif addon and not select(2, GetAddOnInfo(addon)) then
		error(LIB .. ":AddEntry(name, parent, addon) - 'addon' (" .. addon .. ") not found", 2)
	end
	local entry = setmetatable({
		addon = not IsAddOnLoaded(addon) and addon or nil,
		name = name,
		parent = parent
	}, self.emulate_mt)
	entry:SetAttribute('title', name)
	InterfaceOptions_AddCategory(entry)
	return entry
end

--[=[----------------------------------------------------------------------------

LIB:HasEntry(name [, parent])

	Determines if an entry is in Blizzard's interface options panel.

Input:

	name		(string) 	The name of the entry to check for.

	parent	(string)		The name of the parent entry if a child is being looked
								for.

Returns:

	index		(boolean)	Index of the entry or nil if it doesn't exist.

------------------------------------------------------------------------------]=]
lib.HasEntry = lib.HasEntry or function(self, name, parent)
	if type(name) ~= 'string' or (parent and type(parent) ~= 'string') then return end
	for index, entry in self:IterateEntries(parent) do
		if entry.name == name then return index end
	end
end

--[=[----------------------------------------------------------------------------

LIB:IterateEntries(parent)

	Acts as a special case ipairs for Blizzard's INTERFACEOPTIONS_ADDONCATEGORIES
	table.

Input:

	parent	(string)		If a string is provided then all entries returned will be
								children of that parent, otherwise only entries with no
								parent are returned.

Returns:

	index		(number)		The index of the entry.

	entry		(table)		The frame reference for the entry.

------------------------------------------------------------------------------]=]
if not lib.IterateEntries then
	local entries = INTERFACEOPTIONS_ADDONCATEGORIES

	local function iterator(parent, index)
		for index = (index or 0) + 1, #entries do
			if entries[index].parent == parent then
				return index, entries[index]
			end
		end
	end

	function lib:IterateEntries(parent)
		return iterator, type(parent) == 'string' and parent or nil, 0
	end
end

--[=[----------------------------------------------------------------------------

LIB:LoadModule(name)

	Loads an addon.  Any addon that is not enabled will have it's status briefly
	changed for loading and then returned to it's original state.

Input:

	name		(string) 	The addon's name as it appears in the Interface\AddOns\
								folder.

Returns:

	loaded	(boolean)	Whether or not the addon is loaded.

	reason	(string)		If the addon is not loaded then this will be the reason
								why.

------------------------------------------------------------------------------]=]
lib.LoadModule = lib.LoadModule or function(self, name)
	if type(name) ~= 'string' then
		error(LIB .. ":LoadModule(name) - 'name' expected string, got " .. type(name), 2)
	end
	local loaded, reason, _, title, _, enabled = nil, 'MISSING', GetAddOnInfo(name)
	if title then
		loaded = IsAddOnLoaded(name)
		if not loaded then
			if not enabled then
				EnableAddOn(name)
			end
			loaded, reason = LoadAddOn(name)
			if not enabled then
				DisableAddOn(name)
			end
		end
	end
	return loaded, reason
end
