setfenv(1, DockingStation)

local max = max

local _SetScript = utilityFrame.SetScript

plugins = { }
local plugins = plugins

--[[-----------------------------------------------------------------------------
Methods
-------------------------------------------------------------------------------]]
local allowedHandlers = { ['function'] = true, ['nil'] = true }
local allowedScripts = { ['OnClick'] = true, ['OnEnter'] = true, ['OnLeave'] = true, ['OnMouseWheel'] = true }
local ICON_TEXT_BUFFER, outline_types, shadow_distances = 3, { [1] = 'OUTLINE', [2] = 'THICKOUTLINE' }, { [3] = 1, [4] = 2 }

local function HideString(string)
	string:SetText(nil)
	string:SetWidth(0)
	string:Hide()
end

local function ShadowString(string, shadow)
	shadow = shadow or 0
	string:SetShadowColor(0, 0, 0, 0.5)
	string:SetShadowOffset(shadow, -shadow)
end

local methods = {
	['GetScript'] = function(self, script)														-- Stop plugins from directly hooking in and bypassing settings
		if not allowedScripts[script] then return end
		return self["_" .. script]
	end,

	["Refresh"] = function(self, skip)
		local data, icon, label, settings, suffix, text, width = self.data, self.icon, self.label, self.settings, self.suffix, self.text, 0
		local textFixed, textFont, textString = settings.textFixed, settings.textFont, data.value or data.text

		if not skip then
			local height = settings.height * SCREEN_HEIGHT
			self:SetHeight(height)
			icon:SetHeight(settings.iconSize * height)
			icon:SetWidth(settings.iconSize * height)

			vertAdj = settings.suffixVertAdj
			suffix:SetFont(LSM:Fetch('font', settings.suffixFont) or DEFAULT_FONT, settings.suffixSize * height, outline_types[settings.suffixEffect])
			suffix:SetTextColor(settings.suffixColorR, settings.suffixColorG, settings.suffixColorB, settings.suffixColorA)
			suffix:SetPoint('RIGHT', self, 'RIGHT', -1, vertAdj * height)
			ShadowString(suffix, shadow_distances[settings.suffixEffect])

			vertAdj = settings.textVertAdj - vertAdj
			text:SetFont(LSM:Fetch('font', textFont) or DEFAULT_FONT, settings.textSize * height, outline_types[settings.textEffect])
			text:SetTextColor(settings.textColorR, settings.textColorG, settings.textColorB, settings.textColorA)
			text:SetPoint('RIGHT', suffix, 'LEFT', 0, vertAdj * height)
			ShadowString(text, shadow_distances[settings.textEffect])

			vertAdj = settings.labelVertAdj - vertAdj - settings.suffixVertAdj
			label:SetFont(LSM:Fetch('font', settings.labelFont) or DEFAULT_FONT, settings.labelSize * height, outline_types[settings.labelEffect])
			label:SetTextColor(settings.labelColorR, settings.labelColorG, settings.labelColorB, settings.labelColorA)
			label:SetPoint('RIGHT', text, 'LEFT', 0, vertAdj * height)
			ShadowString(label, shadow_distances[settings.labelEffect])

			text:SetHeight(0)
			text:SetHeight(text:GetStringHeight())
			if textFixed then
				text:SetWidth(0)
				text:SetText("...")
				local minWidth = text:GetStringWidth()
				text:SetText(textFixed)
				text:SetWidth(max(minWidth, text:GetStringWidth()))
			end
		end

		if data.icon and not settings.iconHide then
			SetTexCoord(icon, self.data.iconCoords, settings.iconZoom)
			width = width + icon:GetWidth()
			icon:Show()
		else
			icon:Hide()
		end

		if textString and not settings.textHide then
			text:SetText(settings.textOverride and RemoveColorCodes(textString) or textString)
			if not textFixed then
				text:SetWidth(0)
				text:SetWidth(text:GetStringWidth())
			end
			width = width + text:GetWidth()
			text:Show()
			if data.value and data.suffix and not settings.suffixHide then
				suffix:SetText(settings.suffixOverride and RemoveColorCodes(" " .. data.suffix) or " " .. data.suffix)
				suffix:SetWidth(0)
				suffix:SetWidth(suffix:GetStringWidth())
				width = width + suffix:GetWidth()
				suffix:Show()
			else
				HideString(suffix)
			end
		else
			HideString(text)
			HideString(suffix)
		end

		if not settings.labelHide or not (icon:IsShown() or text:IsShown()) then
			if text:IsShown() then
				label:SetText(settings.labelOverride and RemoveColorCodes(data.label .. ": ") or data.label .. ": ")
			else
				label:SetText(settings.labelOverride and RemoveColorCodes(data.label) or data.label)
			end
			label:SetWidth(0)
			label:SetWidth(label:GetStringWidth())
			width = width + label:GetWidth()
			label:Show()
		else
			HideString(label)
		end

		if icon:IsShown() and (text:IsShown() or label:IsShown()) then
			width = width + ICON_TEXT_BUFFER
		end

		self:SetWidth(width)
	end,

	['SetScript'] = function(self, script, handler)											-- Stop plugins from directly hooking in and bypassing settings
		if not (allowedScripts[script] and allowedHandlers[type(handler)]) then return end
		self:EnableMouseWheel(self._OnMouseWheel)
		self["_" .. script] = handler
	end,

	["SetState"] = function(self, override)
		if override then
			self.panel, self.section, self.index = nil, nil, nil
		else
			self.panel, self.section, self.index = GetPluginLocation(self.name)
		end
		if self.settings.enable then
			self.SetWidth = self.section == "Center" and self.panel._SetWidth or nil
			self:Show()
		else
			self.SetWidth = nil
			self:Hide()
		end
		self:SetParent(self.panel or UIParent)
		SetInheritance(self.name, self.panel)
		if self.panel then
			self:SetFrameLevel(self.panel:GetFrameLevel() + 1)
			self.panel:Refresh()
		end
	end
}

local mt_plugin = { __index = setmetatable(methods, MT_FRAME) }

--[[-----------------------------------------------------------------------------
Drag-and-drop support
-------------------------------------------------------------------------------]]
local function ActiveIndexToRealIndex(panel, sectionType, activeIndex)
	local activeCount, lastActive, plugin = 0, 1
	for index, name in ipairs(panel.settings[sectionType]) do
		plugin = plugins[name]
		if plugin and plugin.settings.enable then
			activeCount = activeCount + 1
			lastActive = index + 1
			if activeCount == activeIndex then
				return index
			end
		end
	end
	return lastActive
end

local function GetLocation(self, x, y)
	local panelOrder, panel = settings.panelOrder
	for index = #panelOrder, 1, -1 do
		panel = panels[panelOrder[index]]
		if panel and panel.settings.enable then
			if panel:GetTop() >= y and y >= panel:GetBottom() and panel:GetLeft() <= x and x <= panel:GetRight() then
				local center, left, right, sixth, x0 = panel.Center, panel.Left, panel.Right, panel:GetWidth() / 6, panel:GetLeft()

				local adjust, section, sectionType
				if x < ((center[1] and center[1]:GetLeft() or x0 + sixth * 3) + (left[1] and left[#left]:GetRight() or x0 + sixth)) / 2 then
					adjust, section, sectionType = -1, left, "Left"
				elseif x < ((right[1] and right[#right]:GetLeft() or x0 + sixth * 5) + (center[1] and center[#center]:GetRight() or x0 + sixth * 3)) / 2 then
					adjust, section, sectionType = -1, center, "Center"
				else
					adjust, section, sectionType = 1, right, "Right"
				end

				local index = 1
				for i = #section, 1, -1 do
					if (section[i]:GetCenter() - x) * adjust > 0 then
						index = i + 1
						break
					end
				end

				return panel, sectionType, ActiveIndexToRealIndex(panel, sectionType, index)
			end
		end
	end
end

--[[-----------------------------------------------------------------------------
Scripts
-------------------------------------------------------------------------------]]
local function OnEnter(self, ...)																-- tooltip > OnEnter > OnTooltipShow > tooltiptext
	local settings = self.settings
	self.panel:UpdateVisibility()
	if (settings.tooltipHideCombat and PLAYER_IN_COMBAT) or (settings.tooltipHideNoCombat and not PLAYER_IN_COMBAT) then return end
	StartTooltipDetection()
	local data, tooltip = self.data
	if data.tooltip then
		tooltip = data.tooltip
		tooltip = pcall(tooltip.SetOwner, tooltip, self, 'ANCHOR_NONE') and tooltip
	elseif self._OnEnter or data.OnEnter then
		pcall(self._OnEnter or data.OnEnter, self, ...)
	else
		tooltip = GameTooltip
		tooltip:SetOwner(self, 'ANCHOR_NONE')
		if data.OnTooltipShow then
			pcall(data.OnTooltipShow, tooltip)
		else
			tooltip:SetText(data.label, 1, 1, 1)
			if data.keybind then
				local ok, keyBind = pcall(GetBindingKey, data.keybind)
				if ok and keyBind then
					tooltip:AppendText(NORMAL_FONT_COLOR_CODE .. " (" .. keyBind .. ")|r")
				end
			end
			if data.tooltiptext then
				tooltip:AddLine(data.tooltiptext, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, true)
			end
		end
	end
	if tooltip then
		ConnectAsTooltip(tooltip, self)
		pcall(tooltip.Show, tooltip)
	end
	StopTooltipDetection()
end

local function OnLeave(self, ...)
	local data = self.data
	if data.tooltip then
		pcall(data.tooltip.Hide, data.tooltip)
	elseif self._OnLeave or data.OnLeave then
		pcall(self._OnLeave or data.OnLeave, self, ...)
	else
		GameTooltip:Hide()
	end
	ReleaseTooltip()
	self.panel:UpdateVisibility()
end

local function OnDragStart(self)
	if self.settings.lockPlugin then return end
	_SetScript(self, 'OnEnter', nil)
	OnLeave(self)
	ReleaseTooltip(true)
	tremove_byVal(self.panel.settings[self.section], self.name)
	self.panel:Refresh()
	self.SetWidth = nil
	local hClamp, vClamp = (self:GetWidth() - 1) / 2, (self:GetHeight() - 1) / 2
	self:SetClampedToScreen(true)
	self:SetClampRectInsets(hClamp, -hClamp, -vClamp, vClamp)
	self:SetFrameStrata('TOOLTIP')
	self:StartMoving()
	DRAG_IN_PROGRESS = self
	AllPanels("UpdateVisibility")
end

local function OnDragStop(self)
	if DRAG_IN_PROGRESS ~= self then return end
	DRAG_IN_PROGRESS = false
	AllPanels("UpdateVisibility")
	self:StopMovingOrSizing()
	self:SetClampedToScreen(false)
	local panel, section, index = GetLocation(self, self:GetCenter())
	panel, section, index = panel or self.panel, section or self.section, index or self.index
	panel:AddPlugin(self.name, section, index)
	_SetScript(self, 'OnEnter', OnEnter)
	if MouseIsOver(self) then
		OnEnter(self)
	end
end

local function OnClick(self, ...)
--	StartTooltipDetection()
	local data = self.data
	if self._OnClick or data.OnClick then
		pcall(self._OnClick or data.OnClick, self, ...)
	end
--	StopTooltipDetection()
--	if data.tooltip then
--		pcall(data.tooltip.Hide, data.tooltip)
--	end
--	ReleaseTooltip(true)
end

local function OnMouseWheel(self, ...)
	pcall(self._OnMouseWheel, self, ...)
end

--[[-----------------------------------------------------------------------------
String support
-------------------------------------------------------------------------------]]
local function CreatePluginString(frame, category)
	local string = frame:CreateFontString(nil, 'ARTWORK')
	string:SetFont(DEFAULT_FONT, 12)
	string:SetJustifyH('LEFT')
	string:SetJustifyV('MIDDLE')
	string:SetNonSpaceWrap(false)
	string:SetShadowColor(0, 0, 0, 0.5)
	frame[category] = string
end

--[[-----------------------------------------------------------------------------
Global to addon
-------------------------------------------------------------------------------]]
DRAG_IN_PROGRESS, PluginList = false, { Add = DoNothing, Delete = DoNothing, Refresh = DoNothing, Select = DoNothing }

function CreatePlugin(name, dataObject, type)
	local plugin = CreateFrame('Button', nil, UIParent)									-- Need the 'OnClick' handler (fix for CastSpellByName)
	plugin.icon = plugin:CreateTexture(nil, 'ARTWORK')
	plugin.icon:SetPoint('LEFT', plugin, 'LEFT')
	CreatePluginString(plugin, "suffix")
	plugin.suffix:SetPoint('RIGHT', plugin, 'RIGHT', -1, 0)
	CreatePluginString(plugin, "text")
	plugin.text:SetPoint('RIGHT', plugin.suffix, 'LEFT', 0, 0)
	CreatePluginString(plugin, "label")
	plugin.label:SetPoint('RIGHT', plugin.text, 'LEFT', 0, 0)
	plugin:EnableMouse(true)
	plugin:SetMovable(true)
	plugin:RegisterForClicks('AnyUp')															-- Part of 'OnClick' fix
	plugin:RegisterForDrag('LeftButton')
	plugin:SetScript('OnClick', OnClick)														-- Part of 'OnClick' fix
	plugin:SetScript('OnDragStart', OnDragStart)
	plugin:SetScript('OnDragStop', OnDragStop)
	plugin:SetScript('OnEnter', OnEnter)
	plugin:SetScript('OnLeave', OnLeave)
	plugin:SetScript('OnMouseWheel', OnMouseWheel)

	plugins[name] = setmetatable(plugin, mt_plugin)
	plugin.name = name
	plugin.data = { type = type }
	plugin.settings = GetPluginSettings(name)
	UpdateDataObject(nil, name, 'label')														-- Label defaults to 'name' if nil or white-space
	LDB.attributestorage[dataObject] = LDB.attributestorage[dataObject] or { }		-- Fix for LDB "bug" (or feature?)
	for field, value in LDB:pairs(dataObject) do
		UpdateDataObject(nil, name, field, value)
	end
	plugin.settings.type = type
	plugin:SetState()
	LDB.RegisterCallback(addon, 'LibDataBroker_AttributeChanged_' .. name, UpdateDataObject)
	PluginList:Add(name)
	return plugin
end
