setfenv(1, DockingStation)

DEFAULT_TEMPLATE = "Panel $id"

--[[-----------------------------------------------------------------------------
Local to module
-------------------------------------------------------------------------------]]
local CheckStringDefaults
do
	local exceptions = {
		labelColorA = -1,
		labelColorB = -2,
		labelColorG = -3,
		labelColorR = -4,
		labelEffect = -5,
		labelFont = -6,
		labelOverride = -7,
		labelSize = -8,
		labelVertAdj = -9,
		suffixColorA = 1,
		suffixColorB = 2,
		suffixColorG = 3,
		suffixColorR = 4,
		suffixEffect = 5,
		suffixFont = 6,
		suffixOverride = 7,
		suffixSize = 8,
		suffixVertAdj = 9
	}
	local index = {
		[-1] = "textColorA",
		[-2] = "textColorB",
		[-3] = "textColorG",
		[-4] = "textColorR",
		[-5] = "textEffect",
		[-6] = "textFont",
		[-7] = "textOverride",
		[-8] = "textSize",
		[-9] = "textVertAdj",
		[1] = "textColorA",
		[2] = "textColorB",
		[3] = "textColorG",
		[4] = "textColorR",
		[5] = "textEffect",
		[6] = "textFont",
		[7] = "textOverride",
		[8] = "textSize",
		[9] = "textVertAdj"
	}
	local options = { [false] = "labelUseText", [true] = "suffixUseText" }
	CheckStringDefaults = function(table, key)
		key = exceptions[key]
		if key and table[options[key > 0]] then
			return table[index[key]]
		end
	end
end

local DEFAULT_SETTINGS = {
	alphaCombat = false,
	alphaDefaults = true,
	alphaFaded = 1,
	alphaMouse = false,
	alphaNormal = 1,
	anchor = 'CENTER',
	appearanceDefaults = true,
	bgColorA = 1,
	bgColorB = 0,
	bgColorG = 0,
	bgColorR = 0,
	bgDefaults = true,
	bgOverlay = "None",
	bgOverlayColorA = 1,
	bgOverlayColorB = 1,
	bgOverlayColorG = 1,
	bgOverlayColorR = 1,
	bgOverlayFlip = false,
	bgOverlayFlop = false,
	bgInset = 0,
	bgTexture = 'Solid',
	borderColorA = 1,
	borderColorB = 0.85,
	borderColorG = 0.85,
	borderColorR = 0.85,
	borderDefaults = true,
	borderSize = 11,
	borderTexture = 'None',
	centerConfig = true,
	height = 20 / 768,
	iconDefaults = true,
	iconHide = false,
	iconSize = 0.85,
	iconZoom = 0,
	labelColorA = 1,
	labelColorB = 1,
	labelColorG = 1,
	labelColorR = 1,
	labelDefaults = true,
	labelEffect = 0,
	labelFont = 'Friz Quadrata TT',
	labelHide = false,
	labelOverride = false,
	labelSize = 0.55,
	labelUseText = true,
	labelVertAdj = 0,
	lockPanel = false,
	lockPlugin = false,
	moveBlizzard = false,
	positionDefaults = true,
	report = false,
	rightClickConfig = false,
	screenClamp = 0,
	spacingCenter = 10,
	spacingDefaults = true,
	spacingLeftEdge = 5,
	spacingLeft = 10,
	spacingRight = 10,
	spacingRightEdge = 5,
	strata = 4,
	suffixColorA = 1,
	suffixColorB = 1,
	suffixColorG = 1,
	suffixColorR = 1,
	suffixDefaults = true,
	suffixEffect = 0,
	suffixFont = 'Friz Quadrata TT',
	suffixHide = false,
	suffixOverride = false,
	suffixSize = 0.55,
	suffixUseText = true,
	suffixVertAdj = 0,
	template = DEFAULT_TEMPLATE,
	textColorA = 1,
	textColorB = 1,
	textColorG = 1,
	textColorR = 1,
	textDefaults = true,
	textEffect = 0,
	textFont = 'Friz Quadrata TT',
	textHide = false,
	textOverride = false,
	textSize = 0.55,
	textVertAdj = 0,
	tooltipDefaults = true,
	tooltipAllowScaling = false,
	tooltipHideCombat = false,
	tooltipHideNoCombat = false,
	tooltipScale = 0.8,
	width = 1
}

local function GetDefaultPanelSettings()
	return {
		alphaDefaults = true,
		appearanceDefaults = true,
		bgDefaults = true,
		borderDefaults = true,
		Center = { },
		enable = true,
		iconDefaults = true,
		labelDefaults = true,
		Left = { },
		offsetX = 0,
		offsetY = 0,
		positionDefaults = true,
		Right = { },
		spacingDefaults = true,
		suffixDefaults = true,
		textDefaults = true,
		tooltipDefaults = true
	}
end

local function GetDefaultPluginSettings()
	return {
		enable = false,
		iconDefaults = true,
		labelDefaults = true,
		suffixDefaults = true,
		textDefaults = true,
		tooltipDefaults = true
	}
end

local function GetSettingsMT(defaults)
	return { __index = function(value, index)
		value = CheckStringDefaults(value, index)
		if value == nil then
			return defaults[index]
		end
		return value
	end }
end

local mt_defaults, mt_panels, mt_settings = GetSettingsMT(DEFAULT_SETTINGS), { }

--[[-----------------------------------------------------------------------------
Global to addon
-------------------------------------------------------------------------------]]
function GetSettings()
	settings = addon.db.profile
	mt_panels, mt_settings = wipe(mt_panels), GetSettingsMT(settings)
	if not settings.panels then
		settings.panels = { [1] = GetDefaultPanelSettings() }
		settings.panels[1].Left[1] = configName
	end
	for id = 1, #settings.panels do
		GetPanelSettings(id)
	end
	if not settings.plugins then
		settings.plugins = { [configName] = GetDefaultPluginSettings() }
		settings.plugins[configName].enable = true
	end
	for name in pairs(settings.plugins) do
		GetPluginSettings(name)
	end
	if not settings.panelOrder then
		settings.panelOrder = { }
		for id = 1, #settings.panels do
			settings.panelOrder[id] = id
		end
	end
	return setmetatable(settings, mt_defaults)
end

function GetPanelSettings(id)
	settings.panels[id] = settings.panels[id] or GetDefaultPanelSettings()
	return setmetatable(settings.panels[id], mt_settings)
end

function GetPluginSettings(name)
	settings.plugins[name] = settings.plugins[name] or GetDefaultPluginSettings()
	return SetInheritance(name, GetPluginLocation(name))
end

function SetInheritance(name, panel)
	if panel then
		mt_panels[panel] = mt_panels[panel] or GetSettingsMT(panel.settings)
		return setmetatable(settings.plugins[name], mt_panels[panel])
	else
		return setmetatable(settings.plugins[name], mt_settings)
	end
end
