setfenv(1, DockingStation)

local GetMouseFocus, MouseIsOver, _GetScript = GetMouseFocus, MouseIsOver, utilityFrame.GetScript

--[[-----------------------------------------------------------------------------
Local to module
-------------------------------------------------------------------------------]]
local owner, parent, savedClamp, savedMT, savedScale, tooltip

local delayRelease, timer = CreateFrame('Frame', nil, UIParent)
delayRelease:SetScript('OnHide', function(self)
	timer = 0
end)
delayRelease:SetScript('OnUpdate', function(self, elapsed)
	timer = timer + elapsed
	if timer < 0.25 then return end
	self:Hide()
	ReleaseTooltip()
end)
delayRelease:Hide()

local function RestoreTooltip()
	pcall(setmetatable, tooltip, savedMT)
	tooltip:SetClampedToScreen(savedClamp)
	tooltip:SetScale(savedScale)
	if tooltip:IsVisible() then
		tooltip:Hide()
	end
	TOOLTIP_ON_PANEL = false
	(parent.panel or parent):UpdateVisibility()
	parent, tooltip = nil, nil
	delayRelease:Hide()
end

local mt_override_mt = { __index = DoNothing }
local mt_override = { __index = setmetatable({
	['ClearAllPoints'] = DoNothing,

	['SetClampedToScreen'] = function(self, value)
		savedClamp = value
	end,

	['SetOwner'] = function(self)
		self:ClearLines()
	end,

	['SetPoint'] = DoNothing,

	['SetScale'] = function(self, value)
		savedScale = value
		if not parent.settings.tooltipAllowScaling then
			savedMT.__index['SetScale'](tooltip, value)
		end
	end
},	mt_override_mt)}

local delayOverride, pendingParent, pendingTooltip = CreateFrame('Frame', nil, UIParent)
delayOverride:SetScript('OnUpdate', function(self)
	if pendingTooltip:IsVisible() then
		if tooltip then
			RestoreTooltip()
		end
		parent, tooltip, TOOLTIP_ON_PANEL = pendingParent, pendingTooltip, pendingParent.panel or pendingParent
		savedClamp, savedMT, savedScale = tooltip:IsClampedToScreen(), getmetatable(tooltip), tooltip:GetScale()
		tooltip:ClearAllPoints()
		ConnectAsTooltip(tooltip, parent)
		tooltip:SetClampedToScreen(true)
		pcall(setmetatable, tooltip, mt_override)
		mt_override_mt.__index = savedMT.__index
		if parent.settings.tooltipAllowScaling then
			savedMT.__index['SetScale'](tooltip, parent.settings.tooltipScale)
			pcall(_GetScript(parent, 'OnEnter'), parent, true)								-- Font strings don't scale well after the fact so draw again with
		end																								--		proper scaling already in place
	end
	pendingParent, pendingTooltip = nil, nil
	self:Hide()
end)
delayOverride:Hide()

local function OverrideTooltip(frame, plugin)
	if plugin == parent then
		if frame ~= tooltip then
			frame:Hide()
		end
		return
	end
	pendingParent, pendingTooltip = plugin, frame
	delayOverride:Show()
end

--[[-----------------------------------------------------------------------------
Hooks
-------------------------------------------------------------------------------]]
hooksecurefunc(getmetatable(GameTooltip).__index, 'SetOwner', function(self, frame, anchor)	-- Try and catch 'OnEnter' tooltips
	if not owner or frame ~= owner then return end
	if anchor ~= 'ANCHOR_NONE' then
		self:SetOwner(frame, 'ANCHOR_NONE')
	end
	OverrideTooltip(self, frame)
end)

hooksecurefunc(getmetatable(delayOverride).__index, 'SetPoint', function(self, _, frame)		-- Try and catch 'OnEnter' custom frames
	if not owner or frame ~= owner then return end
	OverrideTooltip(self, frame)
end)

--[[-----------------------------------------------------------------------------
Global to addon
-------------------------------------------------------------------------------]]
TOOLTIP_ON_PANEL = false

function ReleaseTooltip(hide)
	if not tooltip then return end
	if tooltip:IsVisible() and (MouseIsOver(tooltip) or (parent ~= UIParent and GetMouseFocus() == parent)) and not hide then
		delayRelease:Show()
	else
		RestoreTooltip()
	end
end

function StartTooltipDetection()
	owner = GetMouseFocus()
end

function StopTooltipDetection()
	owner = nil
end
