local loadTime


-- the frame itself
local optionsFrame = CreateFrame("Frame", nil, UIParent)
optionsFrame.name = "Ghost: Pulse"
optionsFrame:SetWidth(350)
optionsFrame:SetHeight(400)

optionsFrame.okay = function()
end

-- function to sync the frame with the settings
local function ShowCurrentSettings()
end

optionsFrame.cancel = function()
    ShowCurrentSettings()
end

optionsFrame.default = function()	
    ShowCurrentSettings()
end

-- config
local function CreateComboBox(name, parent)
	local rc
	
	
	rc = CreateFrame("Button", name, parent, "UIDropDownMenuTemplate")
	
	function rc:init()
		if rc.list ~= nil then
			local info = UIDropDownMenu_CreateInfo()			
			local items
			
			
			if type(rc.list) == "function" then
				items = rc.list()
			else
				items = rc.list
			end
			
			table.sort(items)
	
			for i, v in pairs(items) do
				info.text = v
				info.value = i
				info.func = function()
					UIDropDownMenu_SetSelectedValue(self, i)
					
					if self.OnItemClick then
						self.OnItemClick(self, v)
					end
				end
				info.checked = nil
				info.icon = nil
				UIDropDownMenu_AddButton(info, 1)
			end
		end
	end
	
	UIDropDownMenu_Initialize(rc, rc.init)

	return rc
end

-- filter management
local function AddToFilterList(this, frameName, item)
	if frameName and item then
		if GhostPulse3.currentProfile.frames[frameName] then
			local suffix
			
			
			if GhostPulse3.currentProfile.frames[frameName].filterType == 1 then
				suffix = "and will not be shown"
			else
				suffix = "and will be shown"
			end
			
			if not GhostPulse3.currentProfile.frames[frameName].filterList[item] then
				GhostPulse3.currentProfile.frames[frameName].filterCount = GhostPulse3.currentProfile.frames[frameName].filterCount + 1
				GhostPulse3.currentProfile.frames[frameName].filterList[item] = true

				GhostPulse3:TellUser(string.format("%s was added to the filter list for %s, %s.", item, frameName, suffix))
			else
				GhostPulse3:TellUser(string.format("%s is already on the filter list for %s, %s.", item, frameName, suffix))
			end
		end
	end
end

local function RemoveFromFilterList(this, frameName, item)
	if frameName and item then
		if GhostPulse3.currentProfile.frames[frameName] then
			local suffix
			
			
			if GhostPulse3.currentProfile.frames[frameName].filterType == 1 then
				suffix = "and will be shown"
			else
				suffix = "and will not be shown"
			end
			
			if GhostPulse3.currentProfile.frames[frameName].filterList[item] then
				GhostPulse3.currentProfile.frames[frameName].filterCount = GhostPulse3.currentProfile.frames[frameName].filterCount - 1
				GhostPulse3.currentProfile.frames[frameName].filterList[item] = nil

				GhostPulse3:TellUser(string.format("%s was removed from the filter list for %s, %s.", item, frameName, suffix))
			else
				GhostPulse3:TellUser(string.format("%s is not on the filter list for %s, %s.", item, frameName, suffix))
			end
		end
	end
end

local function ClearFilterList(this, frameName)
	if frameName then
		if GhostPulse3.currentProfile.frames[frameName] then
			GhostPulse3.currentProfile.frames[frameName].filterCount = 0
			GhostPulse3.currentProfile.frames[frameName].filterList = { }
		end
	end
end

local function ShowFilterList(this, frameName)
	if frameName then
		if GhostPulse3.currentProfile.frames[frameName] then
			local data = { }
			
			
			if GhostPulse3.currentProfile.frames[frameName].filterCount == 0 then
				if GhostPulse3.currentProfile.frames[frameName].filterType == 0 then
					GhostPulse3:TellUser("The list is empty, and so the "..frameName.." frame will not show anything.")
				else
					GhostPulse3:TellUser("The list is empty, and so the "..frameName.." frame will not show everything.")
				end				
			else
				if GhostPulse3.currentProfile.frames[frameName].filterType == 0 then
					GhostPulse3:TellUser("Only the things below will be shown in the "..frameName.." frame...")
				else
					GhostPulse3:TellUser("Everything *except* the things below will be shown in the "..frameName.." frame...")
				end
						
				for name, _ in pairs(GhostPulse3.currentProfile.frames[frameName].filterList) do
					table.insert(data, name)
				end
				
				table.sort(data)
				
				for _, name in pairs(data) do
					GhostPulse3:TellUser("    - "..name)
				end
			end
		end
	end
end

-- pulse config frame creation
local function CreateConfigFrame(name)
	local rc
	local f = getglobal("GhostPulseFrameConfig_"..name)

	
	if f == nil then
		local xPos, yPos = 0, 0
		
		
		rc = CreateFrame("Frame", "GhostPulseFrameConfig_"..name, UIParent)
		
		rc.name = "F: "..name
		rc.parent = optionsFrame.name
		
		-- title
		rc.Title = rc:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge")
		rc.Title:SetWidth(340)
		rc.Title:SetHeight(20)
		rc.Title:SetPoint("TOPLEFT", 15, -14)
		rc.Title:SetJustifyH("LEFT")
		rc.Title:SetText(name.." Frame")
	
		-- title
		rc.AlertsTitle = rc:CreateFontString(nil, "OVERLAY", "GameFontNormal")
		rc.AlertsTitle:SetWidth(340)
		rc.AlertsTitle:SetHeight(20)
		rc.AlertsTitle:SetPoint("TOPLEFT", 12, -88)
		rc.AlertsTitle:SetJustifyH("LEFT")
		rc.AlertsTitle:SetText("Which alerts should show in this frame?")
	
		-- remove frame button
		rc.RemoveFrame = CreateFrame("Button", nil, rc, "UIPanelButtonTemplate")
		rc.RemoveFrame:SetScript("OnClick", function()
			StaticPopup_Show("GhostPulseRemoveFrameConfirm_"..name)
		end)
		rc.RemoveFrame:RegisterForClicks("LeftButtonUp")
		rc.RemoveFrame:SetPoint("TOPRIGHT", rc, "TOPRIGHT", -8, -12)
		rc.RemoveFrame:EnableMouse()
		rc.RemoveFrame:SetText("Remove")
		rc.RemoveFrame:SetWidth(112)
		rc.RemoveFrame:SetHeight(24)

		-- filters title
		rc.FiltersTitle = rc:CreateFontString(nil, "OVERLAY", "GameFontNormal")
		rc.FiltersTitle:SetWidth(340)
		rc.FiltersTitle:SetHeight(20)
		rc.FiltersTitle:SetPoint("BOTTOMLEFT", 16, 52)
		rc.FiltersTitle:SetJustifyH("LEFT")
		rc.FiltersTitle:SetText("Filters")
	
		-- filter type title
		rc.FilterTypeTitle = rc:CreateFontString(nil, "OVERLAY", "GameFontNormal")
		rc.FilterTypeTitle:SetWidth(340)
		rc.FilterTypeTitle:SetHeight(20)
		rc.FilterTypeTitle:SetPoint("BOTTOMLEFT", 220, 52)
		rc.FilterTypeTitle:SetJustifyH("LEFT")
		rc.FilterTypeTitle:SetTextColor(1, 1, 1)
		rc.FilterTypeTitle:SetText("Type")
	
		-- filter type
		rc.FilterType = CreateComboBox("GhostPulseFilterType_"..name, rc)
		rc.FilterType.list = { "Blacklist", "Whitelist" }
		rc.FilterType:SetPoint("BOTTOMLEFT", 248, 42)
		rc.FilterType.OnItemClick = function(menu, item)
			if item == "Blacklist" then
				GhostPulse3.currentProfile.frames[name].filterType = 1
				
			elseif item == "Whitelist" then
				GhostPulse3.currentProfile.frames[name].filterType = 0
			end
		end

		if GhostPulse3.currentProfile.frames[name].filterType == 0 then
			UIDropDownMenu_SetText(rc.FilterType, "Whitelist")

		elseif GhostPulse3.currentProfile.frames[name].filterType == 1 then
			UIDropDownMenu_SetText(rc.FilterType, "Blacklist")
		end

		-- add to filter list
		rc.AddToFilterList = CreateFrame("Button", nil, rc, "UIPanelButtonTemplate")
		rc.AddToFilterList:SetScript("OnClick", function()
			StaticPopup_Show("GhostPulseFilterAdd_"..name)
		end)
		rc.AddToFilterList:RegisterForClicks("LeftButtonUp")
		rc.AddToFilterList:SetPoint("BOTTOMLEFT", rc, "BOTTOMLEFT", 12, 16)
		rc.AddToFilterList:EnableMouse()
		rc.AddToFilterList:SetText("Add...")
		rc.AddToFilterList:SetWidth(92)
		rc.AddToFilterList:SetHeight(24)

		-- remove from filter list
		rc.RemoveFromFilterList = CreateFrame("Button", nil, rc, "UIPanelButtonTemplate")
		rc.RemoveFromFilterList:SetScript("OnClick", function()
			StaticPopup_Show("GhostPulseFilterRemove_"..name)
		end)
		rc.RemoveFromFilterList:RegisterForClicks("LeftButtonUp")
		rc.RemoveFromFilterList:SetPoint("BOTTOMLEFT", rc, "BOTTOMLEFT", 112, 16)
		rc.RemoveFromFilterList:EnableMouse()
		rc.RemoveFromFilterList:SetText("Remove...")
		rc.RemoveFromFilterList:SetWidth(92)
		rc.RemoveFromFilterList:SetHeight(24)

		-- show filter list
		rc.ShowFilterList = CreateFrame("Button", nil, rc, "UIPanelButtonTemplate")
		rc.ShowFilterList:SetScript("OnClick", function()
			GhostPulse3:ShowFilterList(name)
		end)
		rc.ShowFilterList:RegisterForClicks("LeftButtonUp")
		rc.ShowFilterList:SetPoint("BOTTOMLEFT", rc, "BOTTOMLEFT", 212, 16)
		rc.ShowFilterList:EnableMouse()
		rc.ShowFilterList:SetText("Show")
		rc.ShowFilterList:SetWidth(92)
		rc.ShowFilterList:SetHeight(24)

		-- clear filter list
		rc.ClearFilterList = CreateFrame("Button", nil, rc, "UIPanelButtonTemplate")
		rc.ClearFilterList:SetScript("OnClick", function()
			StaticPopup_Show("GhostPulseClearFilters_"..name)
		end)
		rc.ClearFilterList:RegisterForClicks("LeftButtonUp")
		rc.ClearFilterList:SetPoint("BOTTOMLEFT", rc, "BOTTOMLEFT", 308, 16)
		rc.ClearFilterList:EnableMouse()
		rc.ClearFilterList:SetText("Clear")
		rc.ClearFilterList:SetWidth(92)
		rc.ClearFilterList:SetHeight(24)

		-- animation combo		
		rc.Animation = CreateComboBox("GhostPulseFrameAnimation_"..name, rc)
		rc.Animation.list = { "Pulse", "Zoom In", "Zoom Out" }
		rc.Animation.OnItemClick = function(menu, item)
			local f = getglobal("GhostPulseFrame_"..name)
			
			
			if f then
				f.animType = item
			end
			
			GhostPulse3.currentProfile.frames[name].animType = item
		end
		rc.Animation:SetPoint("TOPLEFT", 144, -50)
		rc.AnimationLabel = rc:CreateFontString(nil, "OVERLAY", "GameFontNormal")
		rc.AnimationLabel:SetWidth(80)
		rc.AnimationLabel:SetHeight(20)
		rc.AnimationLabel:SetPoint("TOPLEFT", 144 - 75, -50 - 3)
		rc.AnimationLabel:SetJustifyH("RIGHT")
		rc.AnimationLabel:SetTextColor(1, 1, 1)
		rc.AnimationLabel:SetText("Animation")

		UIDropDownMenu_SetText(rc.Animation, GhostPulse3.currentProfile.frames[name].animType or "Pulse")

		for id, info in pairs(GhostPulse3.categories) do
			local catBox = CreateFrame("CheckButton", nil, rc, "OptionsCheckButtonTemplate")
			local catLabel = rc:CreateFontString(nil, "OVERLAY", "GameFontNormal")
			
			
			catBox:SetPoint("TOPLEFT", 8 + xPos * 192, -(yPos * 24 + 110))
			catBox:SetWidth(24)
			catBox:SetHeight(24)
			catBox:SetScript("OnClick", function()
				if catBox:GetChecked() then
					GhostPulse3.currentProfile.frames[name].categories[id] = true
				else
					GhostPulse3.currentProfile.frames[name].categories[id] = nil
				end
			end)
			
			catLabel:SetWidth(300)
			catLabel:SetHeight(20)
			catLabel:SetPoint("TOPLEFT", 32 + xPos * 192, -(yPos * 24 + 2 + 110))
			catLabel:SetJustifyH("LEFT")
			catLabel:SetTextColor(1, 1, 1)
			catLabel:SetText(info.name)
			
			catBox.categoryId = id
			
			if GhostPulse3.currentProfile.frames[name].categories[id] then
				catBox:SetChecked(true)
			end
			
			xPos = xPos + 1
			
			if xPos == 2 then
				xPos = 0	
				yPos = yPos + 1
			end
		end
		
		StaticPopupDialogs["GhostPulseFilterAdd_"..name] = {
			text = "Add item to the filter list for "..name.."...",
			button1 = "Okay",
			button2 = "Cancel",
			whileDead = 1,
			hideOnEscape = 1,
			timeout = 0,
			hasEditBox = 1,
			OnAccept = function()
				local box = getglobal(this:GetParent():GetName().."EditBox")
				local itemName = box:GetText()

				
				if itemName ~= nil and string.len(itemName) > 0 then
					GhostPulse3:AddToFilterList(name, itemName)
				end

				box:SetText("")
			end,
		}
	
		StaticPopupDialogs["GhostPulseFilterRemove_"..name] = {
			text = "Remove item from the filter list for "..name.."...",
			button1 = "Okay",
			button2 = "Cancel",
			whileDead = 1,
			hideOnEscape = 1,
			timeout = 0,
			hasEditBox = 1,
			OnAccept = function()
				local box = getglobal(this:GetParent():GetName().."EditBox")
				local itemName = box:GetText()
				
				
				if itemName ~= nil and string.len(itemName) > 0 then
					GhostPulse3:RemoveFromFilterList(name, itemName)
				end
				
				box:SetText("")
			end,
		}
	
		StaticPopupDialogs["GhostPulseClearFilters_"..name] = {
			text = "Do you really want to clear the filters for "..name.."?",
			button1 = "Yes",
			button2 = "No",
			whileDead = 1,
			hideOnEscape = 1,
			timeout = 0,
			OnAccept = function()
				local suffix
				
				
				GhostPulse3.currentProfile.frames[name].filterList = { }
				GhostPulse3.currentProfile.frames[name].filterCount = 0
				
				if GhostPulse3.currentProfile.frames[name].filterType == 0 then
					suffix = "Nothing will be shown in this frame because it is a whitelist."
				else
					suffix = "Everything will be shown in this frame because it is a blacklist."
				end
				
				GhostPulse3:TellUser(string.format("The filter list for %s has been cleared.  %s", name, suffix))
			end,
		}

		StaticPopupDialogs["GhostPulseRemoveFrameConfirm_"..name] = {
			text = "Do you really want to remove the '"..name.."' pulse frame?",
			button1 = "Yes",
			button2 = "No",
			whileDead = 1,
			hideOnEscape = 1,
			timeout = 0,
			OnAccept = function()
				local f = getglobal("GhostPulseFrame_"..name)
				
				
				if f then
					f.disabled = true
					f:SetLocked(1)
				end
				
				GhostPulse3.currentProfile.frames[name] = nil
				
				GhostPulse3:TellUser("You will need to reload your UI for this change to take effect.")
			end,
		}
	end

	return rc
end

-- pulse config frame creation
local function CreateAlertConfigFrame(name, id)
	local rc
	local f = getglobal("GhostPulseAlertConfig_"..name)

	
	if f == nil then
		local xPos, yPos = 0, 0
		
		
		rc = CreateFrame("Frame", "GhostPulseAlertConfig_"..name, UIParent)
		
		rc.name = "A: "..name
		rc.parent = optionsFrame.name
		rc.categoryId = id
		
		-- title
		rc.Title = rc:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge")
		rc.Title:SetWidth(340)
		rc.Title:SetHeight(20)
		rc.Title:SetPoint("TOPLEFT", 15, -14)
		rc.Title:SetJustifyH("LEFT")
		rc.Title:SetText(name.." Alert")

		-- solo check-box
		rc.WhileSolo = CreateFrame("CheckButton", nil, rc, "OptionsCheckButtonTemplate")
		rc.WhileSolo:SetPoint("TOPLEFT", rc, "TOPLEFT", 12, -50)
		rc.WhileSolo:SetWidth(24)
		rc.WhileSolo:SetHeight(24)
		rc.WhileSolo:SetChecked(true)
		-- solo label
		rc.WhileSoloLabel = rc:CreateFontString(nil, "OVERLAY", "GameFontNormal")
		rc.WhileSoloLabel:SetWidth(300)
		rc.WhileSoloLabel:SetHeight(20)
		rc.WhileSoloLabel:SetPoint("TOPLEFT", 40, -51)
		rc.WhileSoloLabel:SetJustifyH("LEFT")
		rc.WhileSoloLabel:SetTextColor(1, 1, 1)
		rc.WhileSoloLabel:SetText("While Solo")
		rc.WhileSolo:SetScript("OnClick", function()
			GhostPulse3.currentProfile.categories[id].whileSolo = rc.WhileSolo:GetChecked()
		end)

		-- party check-box
		rc.WhileParty = CreateFrame("CheckButton", nil, rc, "OptionsCheckButtonTemplate")
		rc.WhileParty:SetPoint("TOPLEFT", rc, "TOPLEFT", 144, -50)
		rc.WhileParty:SetWidth(24)
		rc.WhileParty:SetHeight(24)
		rc.WhileParty:SetChecked(true)
		-- party label
		rc.WhilePartyLabel = rc:CreateFontString(nil, "OVERLAY", "GameFontNormal")
		rc.WhilePartyLabel:SetWidth(300)
		rc.WhilePartyLabel:SetHeight(20)
		rc.WhilePartyLabel:SetPoint("TOPLEFT", 172, -51)
		rc.WhilePartyLabel:SetJustifyH("LEFT")
		rc.WhilePartyLabel:SetTextColor(1, 1, 1)
		rc.WhilePartyLabel:SetText("While Partied")
		rc.WhileParty:SetScript("OnClick", function()
			GhostPulse3.currentProfile.categories[id].whileParty = rc.WhileParty:GetChecked()
		end)

		-- raid check-box
		rc.WhileRaid = CreateFrame("CheckButton", nil, rc, "OptionsCheckButtonTemplate")
		rc.WhileRaid:SetPoint("TOPLEFT", rc, "TOPLEFT", 276, -50)
		rc.WhileRaid:SetWidth(24)
		rc.WhileRaid:SetHeight(24)
		rc.WhileRaid:SetChecked(true)
		-- raid label
		rc.WhileRaidLabel = rc:CreateFontString(nil, "OVERLAY", "GameFontNormal")
		rc.WhileRaidLabel:SetWidth(300)
		rc.WhileRaidLabel:SetHeight(20)
		rc.WhileRaidLabel:SetPoint("TOPLEFT", 304, -51)
		rc.WhileRaidLabel:SetJustifyH("LEFT")
		rc.WhileRaidLabel:SetTextColor(1, 1, 1)
		rc.WhileRaidLabel:SetText("While in a raid")
		rc.WhileRaid:SetScript("OnClick", function()
			GhostPulse3.currentProfile.categories[id].whileRaid = rc.WhileRaid:GetChecked()
		end)

		-- dead check-box
		rc.WhileDead = CreateFrame("CheckButton", nil, rc, "OptionsCheckButtonTemplate")
		rc.WhileDead:SetPoint("TOPLEFT", rc, "TOPLEFT", 12, -82)
		rc.WhileDead:SetWidth(24)
		rc.WhileDead:SetHeight(24)
		rc.WhileDead:SetChecked(true)
		-- dead label
		rc.WhileDeadLabel = rc:CreateFontString(nil, "OVERLAY", "GameFontNormal")
		rc.WhileDeadLabel:SetWidth(300)
		rc.WhileDeadLabel:SetHeight(20)
		rc.WhileDeadLabel:SetPoint("TOPLEFT", 40, -83)
		rc.WhileDeadLabel:SetJustifyH("LEFT")
		rc.WhileDeadLabel:SetTextColor(1, 1, 1)
		rc.WhileDeadLabel:SetText("While Dead")
		rc.WhileDead:SetScript("OnClick", function()
			GhostPulse3.currentProfile.categories[id].whileDead = rc.WhileDead:GetChecked()
		end)

		-- in combat check-box
		rc.InCombat = CreateFrame("CheckButton", nil, rc, "OptionsCheckButtonTemplate")
		rc.InCombat:SetPoint("TOPLEFT", rc, "TOPLEFT", 144, -82)
		rc.InCombat:SetWidth(24)
		rc.InCombat:SetHeight(24)
		rc.InCombat:SetChecked(true)
		-- in combat label
		rc.InCombatLabel = rc:CreateFontString(nil, "OVERLAY", "GameFontNormal")
		rc.InCombatLabel:SetWidth(300)
		rc.InCombatLabel:SetHeight(20)
		rc.InCombatLabel:SetPoint("TOPLEFT", 172, -83)
		rc.InCombatLabel:SetJustifyH("LEFT")
		rc.InCombatLabel:SetTextColor(1, 1, 1)
		rc.InCombatLabel:SetText("In combat")
		rc.InCombat:SetScript("OnClick", function()
			GhostPulse3.currentProfile.categories[id].inCombat = rc.InCombat:GetChecked()
		end)

		-- out of combat check-box
		rc.OutOfCombat = CreateFrame("CheckButton", nil, rc, "OptionsCheckButtonTemplate")
		rc.OutOfCombat:SetPoint("TOPLEFT", rc, "TOPLEFT", 276, -82)
		rc.OutOfCombat:SetWidth(24)
		rc.OutOfCombat:SetHeight(24)
		rc.OutOfCombat:SetChecked(true)
		-- out of combat label
		rc.OutOfCombatLabel = rc:CreateFontString(nil, "OVERLAY", "GameFontNormal")
		rc.OutOfCombatLabel:SetWidth(300)
		rc.OutOfCombatLabel:SetHeight(20)
		rc.OutOfCombatLabel:SetPoint("TOPLEFT", 304, -83)
		rc.OutOfCombatLabel:SetJustifyH("LEFT")
		rc.OutOfCombatLabel:SetTextColor(1, 1, 1)
		rc.OutOfCombatLabel:SetText("Out of combat")
		rc.OutOfCombat:SetScript("OnClick", function()
			GhostPulse3.currentProfile.categories[id].outOfCombat = rc.OutOfCombat:GetChecked()
		end)

		-- early warnings check-box
		rc.EarlyWarning = CreateFrame("CheckButton", nil, rc, "OptionsCheckButtonTemplate")
		rc.EarlyWarning:SetPoint("TOPLEFT", rc, "TOPLEFT", 12, -132)
		rc.EarlyWarning:SetWidth(24)
		rc.EarlyWarning:SetHeight(24)
		rc.EarlyWarning:SetChecked(true)
		rc.EarlyWarning:SetScript("OnClick", function()
			GhostPulse3.currentProfile.categories[id].earlyWarning = rc.EarlyWarning:GetChecked()
		end)
		-- early warning label
		rc.EarlyWarningLabel = rc:CreateFontString(nil, "OVERLAY", "GameFontNormal")
		rc.EarlyWarningLabel:SetWidth(300)
		rc.EarlyWarningLabel:SetHeight(20)
		rc.EarlyWarningLabel:SetPoint("TOPLEFT", 40, -133)
		rc.EarlyWarningLabel:SetJustifyH("LEFT")
		rc.EarlyWarningLabel:SetTextColor(1, 1, 1)
		rc.EarlyWarningLabel:SetText("Early Warning")

		-- 'early warning time' slider
		rc.EarlyWarningTime = CreateFrame("Slider", nil, rc, "OptionsSliderTemplate")
		rc.EarlyWarningTime:SetPoint("TOPLEFT", rc, "TOPLEFT", 140, -136)
		rc.EarlyWarningTime:SetWidth(256)
		rc.EarlyWarningTime:SetHeight(16)
		rc.EarlyWarningTime:SetOrientation("HORIZONTAL")
		rc.EarlyWarningTime:SetMinMaxValues(0.5, 10)
		rc.EarlyWarningTime:SetValueStep(0.5)
		rc.EarlyWarningTime:SetScript("OnValueChanged", function()
			GhostPulse3.currentProfile.categories[id].earlyWarningTime = rc.EarlyWarningTime:GetValue()
			rc.EarlyWarningTimeLabel:SetText(string.format("%0.1f seconds", rc.EarlyWarningTime:GetValue()))
		end)
		-- early warning time label
		rc.EarlyWarningTimeLabel = rc:CreateFontString(nil, "OVERLAY", "GameFontNormal")
		rc.EarlyWarningTimeLabel:SetWidth(260)
		rc.EarlyWarningTimeLabel:SetHeight(20)
		rc.EarlyWarningTimeLabel:SetPoint("TOPLEFT", 136, -120)
		rc.EarlyWarningTimeLabel:SetJustifyH("CENTER")
		rc.EarlyWarningTimeLabel:SetTextColor(1, 1, 1)
		rc.EarlyWarningTimeLabel:SetText("Early Warning Time")
		
		rc.EarlyWarningMethod = CreateComboBox("EarlyWarningMethod_"..id, rc)
		rc.EarlyWarningMethod.list = { "Both", "Pulse", "Text" }
		rc.EarlyWarningMethod.OnItemClick = function(menu, item)
			GhostPulse3.currentProfile.categories[id].earlyWarningMethod = item
		end
		rc.EarlyWarningMethod:SetPoint("TOPLEFT", 124, -168)
		rc.EarlyWarningMethodLabel = rc:CreateFontString(nil, "OVERLAY", "GameFontNormal")
		rc.EarlyWarningMethodLabel:SetWidth(80)
		rc.EarlyWarningMethodLabel:SetHeight(20)
		rc.EarlyWarningMethodLabel:SetPoint("TOPLEFT", 124 - 75, -168 - 3)
		rc.EarlyWarningMethodLabel:SetJustifyH("RIGHT")
		rc.EarlyWarningMethodLabel:SetTextColor(1, 1, 1)
		rc.EarlyWarningMethodLabel:SetText("Method")

		-- sound
		rc.Sound = CreateFrame("EditBox", "GhostPulseSound_"..id, rc, "InputBoxTemplate")
		rc.Sound:SetPoint("TOPLEFT", 140, -208)
		rc.Sound:SetTextInsets(0, 0, 0, 0)
		rc.Sound:SetWidth(256)
		rc.Sound:SetHeight(32)
		rc.Sound:SetAutoFocus(false)

		-- sound label
		rc.SoundLabel = rc:CreateFontString(nil, "OVERLAY", "GameFontNormal")
		rc.SoundLabel:SetWidth(112)
		rc.SoundLabel:SetHeight(20)
		rc.SoundLabel:SetPoint("TOPLEFT", 12, -214)
		rc.SoundLabel:SetJustifyH("RIGHT")
		rc.SoundLabel:SetTextColor(1, 1, 1)
		rc.SoundLabel:SetText("Sound")
		
		-- early warning sound
		rc.EarlyWarningSound = CreateFrame("EditBox", "GhostPulseEarlyWarningSound_"..id, rc, "InputBoxTemplate")
		rc.EarlyWarningSound:SetPoint("TOPLEFT", 140, -236)
		rc.EarlyWarningSound:SetTextInsets(0, 0, 0, 0)
		rc.EarlyWarningSound:SetWidth(256)
		rc.EarlyWarningSound:SetHeight(32)
		rc.EarlyWarningSound:SetAutoFocus(false)

		-- early warning sound label
		rc.EarlyWarningSoundLabel = rc:CreateFontString(nil, "OVERLAY", "GameFontNormal")
		rc.EarlyWarningSoundLabel:SetWidth(112)
		rc.EarlyWarningSoundLabel:SetHeight(20)
		rc.EarlyWarningSoundLabel:SetPoint("TOPLEFT", 12, -242)
		rc.EarlyWarningSoundLabel:SetJustifyH("RIGHT")
		rc.EarlyWarningSoundLabel:SetTextColor(1, 1, 1)
		rc.EarlyWarningSoundLabel:SetText("Warning Sound")
		
		-- show text
		rc.ShowText = CreateFrame("CheckButton", nil, rc, "OptionsCheckButtonTemplate")
		rc.ShowText:SetPoint("TOPLEFT", rc, "TOPLEFT", 12, -278)
		rc.ShowText:SetWidth(24)
		rc.ShowText:SetHeight(24)
		rc.ShowText:SetChecked(true)
		-- show text label
		rc.ShowTextLabel = rc:CreateFontString(nil, "OVERLAY", "GameFontNormal")
		rc.ShowTextLabel:SetWidth(300)
		rc.ShowTextLabel:SetHeight(20)
		rc.ShowTextLabel:SetPoint("TOPLEFT", 40, -279)
		rc.ShowTextLabel:SetJustifyH("LEFT")
		rc.ShowTextLabel:SetTextColor(1, 1, 1)
		rc.ShowTextLabel:SetText("Show text")
		rc.ShowText:SetScript("OnClick", function()
			GhostPulse3.currentProfile.categories[id].showText = rc.ShowText:GetChecked()
		end)

--[[
		-- max alpha slider
		rc.MaxAlpha = CreateFrame("Slider", nil, rc, "OptionsSliderTemplate")
		rc.MaxAlpha:SetPoint("TOPLEFT", rc, "TOPLEFT", 140, -320)
		rc.MaxAlpha:SetWidth(256)
		rc.MaxAlpha:SetHeight(16)
		rc.MaxAlpha:SetOrientation("HORIZONTAL")
		rc.MaxAlpha:SetMinMaxValues(0.1, 1.0)
		rc.MaxAlpha:SetValueStep(0.1)
		rc.MaxAlpha:SetScript("OnValueChanged", function()
			GhostPulse3.currentProfile.categories[id].maxAlpha = rc.MaxAlpha:GetValue()
			rc.MaxAlphaLabel:SetText(string.format("Opacity: %0.1f", rc.MaxAlpha:GetValue()))
		end)
		-- max alpha label
		rc.MaxAlphaLabel = rc:CreateFontString(nil, "OVERLAY", "GameFontNormal")
		rc.MaxAlphaLabel:SetWidth(260)
		rc.MaxAlphaLabel:SetHeight(20)
		rc.MaxAlphaLabel:SetPoint("TOPLEFT", 136, -304)
		rc.MaxAlphaLabel:SetJustifyH("CENTER")
		rc.MaxAlphaLabel:SetTextColor(1, 1, 1)
		rc.MaxAlphaLabel:SetText("Opacity: 1.0")
]]--
		-- conditions
		rc.Conditions = CreateFrame("EditBox", "GhostPulseConditions_"..id, rc, "InputBoxTemplate")
		rc.Conditions:SetPoint("BOTTOMLEFT", 140, 10)
		rc.Conditions:SetTextInsets(0, 0, 0, 0)
		rc.Conditions:SetWidth(256)
		rc.Conditions:SetHeight(32)
		rc.Conditions:SetAutoFocus(false)

		-- conditions label
		rc.ConditionsLabel = rc:CreateFontString(nil, "OVERLAY", "GameFontNormal")
		rc.ConditionsLabel:SetWidth(112)
		rc.ConditionsLabel:SetHeight(20)
		rc.ConditionsLabel:SetPoint("BOTTOMLEFT", 12, 16)
		rc.ConditionsLabel:SetJustifyH("RIGHT")
		rc.ConditionsLabel:SetTextColor(1, 1, 1)
		rc.ConditionsLabel:SetText("Conditions")
		
		-- supports early warnings
		if not GhostPulse3.categories[id].supportsEarlyWarnings then
			rc.EarlyWarning:SetChecked(nil)
			rc.EarlyWarning:Disable()
			rc.EarlyWarningTime:Disable()
			rc.EarlyWarning:SetAlpha(0.5)
			rc.EarlyWarningLabel:SetAlpha(0.5)
			rc.EarlyWarningTime:SetAlpha(0.5)
			rc.EarlyWarningTimeLabel:SetAlpha(0.5)
			rc.EarlyWarningSoundLabel:SetAlpha(0.5)
			rc.EarlyWarningSound:SetAlpha(0.5)
			rc.EarlyWarningSound:EnableKeyboard(false)
			rc.EarlyWarningSound:EnableMouse(false)
			
			UIDropDownMenu_DisableDropDown(rc.EarlyWarningMethod)
			rc.EarlyWarningMethod:SetAlpha(0.5)
			rc.EarlyWarningMethodLabel:SetAlpha(0.5)
		end
		
		-- prep the database
		GhostPulse3.currentProfile.categories = GhostPulse3.currentProfile.categories or { }
		GhostPulse3.currentProfile.categories[id] = GhostPulse3.currentProfile.categories[id] or { }			

		rc.okay = function()
			GhostPulse3.currentProfile.categories[id].sound = rc.Sound:GetText()
			GhostPulse3.currentProfile.categories[id].earlyWarningSound = rc.EarlyWarningSound:GetText()
			GhostPulse3.currentProfile.categories[id].conditions = rc.Conditions:GetText()
		end
		
		rc:SetScript("OnShow", function()
			rc.Conditions:SetText(GhostPulse3.currentProfile.categories[id].conditions or "")
			rc.Sound:SetText(GhostPulse3.currentProfile.categories[id].sound or "")
			rc.EarlyWarningSound:SetText(GhostPulse3.currentProfile.categories[id].earlyWarningSound or "")
			rc.EarlyWarning:SetChecked(GhostPulse3.currentProfile.categories[id].earlyWarning)
			rc.EarlyWarningTime:SetValue(GhostPulse3.currentProfile.categories[id].earlyWarningTime or 0.5)
			rc.EarlyWarningTimeLabel:SetText(string.format("%0.1f seconds", rc.EarlyWarningTime:GetValue()))
			rc.WhileSolo:SetChecked(GhostPulse3.currentProfile.categories[id].whileSolo)
			rc.WhileParty:SetChecked(GhostPulse3.currentProfile.categories[id].whileParty)
			rc.WhileRaid:SetChecked(GhostPulse3.currentProfile.categories[id].whileRaid)
			rc.WhileDead:SetChecked(GhostPulse3.currentProfile.categories[id].whileDead)
			rc.InCombat:SetChecked(GhostPulse3.currentProfile.categories[id].inCombat)
			rc.OutOfCombat:SetChecked(GhostPulse3.currentProfile.categories[id].outOfCombat)
			rc.ShowText:SetChecked(GhostPulse3.currentProfile.categories[id].showText)
--			rc.MaxAlpha:SetValue(GhostPulse3.currentProfile.categories[id].maxAlpha or 1.0)
--			rc.MaxAlphaLabel:SetText(string.format("Opacity: %0.1f", rc.MaxAlpha:GetValue()))
			UIDropDownMenu_SetText(rc.EarlyWarningMethod, GhostPulse3.currentProfile.categories[id].earlyWarningMethod or "Pulse")
		end)
	end

	return rc
end

-- title
optionsFrame.Title = optionsFrame:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge")
optionsFrame.Title:SetWidth(340)
optionsFrame.Title:SetHeight(20)
optionsFrame.Title:SetPoint("TOPLEFT", 15, -14)
optionsFrame.Title:SetJustifyH("LEFT")
optionsFrame.Title:SetText("Ghost: Pulse "..GetAddOnMetadata("GhostPulse3", "Version"))

-- locked check-box
optionsFrame.Locked = CreateFrame("CheckButton", nil, optionsFrame, "OptionsCheckButtonTemplate")
optionsFrame.Locked:SetPoint("TOPLEFT", optionsFrame, "TOPLEFT", 310, -390)
optionsFrame.Locked:SetWidth(24)
optionsFrame.Locked:SetHeight(24)
optionsFrame.Locked:SetChecked(true)
-- locked label
optionsFrame.LockedLabel = optionsFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
optionsFrame.LockedLabel:SetWidth(300)
optionsFrame.LockedLabel:SetHeight(20)
optionsFrame.LockedLabel:SetPoint("TOPLEFT", 340, -391)
optionsFrame.LockedLabel:SetJustifyH("LEFT")
optionsFrame.LockedLabel:SetTextColor(1, 1, 1)
optionsFrame.LockedLabel:SetText("Locked")
optionsFrame.Locked:SetScript("OnClick", function()
	GhostPulse3:SetLocked(optionsFrame.Locked:GetChecked())
	
	if not optionsFrame.Locked:GetChecked() then
		GhostPulse3:TellUser("Drag the frames to the position you want them.  Use the scroll wheel while over a frame to adjust its size, or use the shift key with the scroll wheel to adjust the text size.")
	end
end)

-- new frame text box
optionsFrame.FrameName = CreateFrame("EditBox", "GhostPulseNewFrameName", optionsFrame, "InputBoxTemplate")
optionsFrame.FrameName:SetPoint("TOPLEFT", 128, -50)
optionsFrame.FrameName:SetTextInsets(0, 0, 0, 0)
optionsFrame.FrameName:SetWidth(152)
optionsFrame.FrameName:SetHeight(32)
optionsFrame.FrameName:SetAutoFocus(false)
optionsFrame.FrameName:SetScript("OnTextChanged", function()
	local allow
	local text = optionsFrame.FrameName:GetText()
	
	
	if text == nil or text == "" then
		allow = nil
		
	elseif GhostPulse3.currentProfile.frames[text] ~= nil then
		allow = nil
		
	else
		allow = true
	end
	
	if allow then
		optionsFrame.CreateFrame:Enable()
	else
		optionsFrame.CreateFrame:Disable()
	end
end)
-- new frame text box label
optionsFrame.FrameNameLabel = optionsFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
optionsFrame.FrameNameLabel:SetWidth(112)
optionsFrame.FrameNameLabel:SetHeight(20)
optionsFrame.FrameNameLabel:SetPoint("TOPLEFT", 4, -56)
optionsFrame.FrameNameLabel:SetJustifyH("RIGHT")
optionsFrame.FrameNameLabel:SetTextColor(1, 1, 1)
optionsFrame.FrameNameLabel:SetText("Frame Name")

-- create new frame button
optionsFrame.CreateFrame = CreateFrame("Button", nil, optionsFrame, "UIPanelButtonTemplate")
optionsFrame.CreateFrame:SetScript("OnClick", function()
	local name = optionsFrame.FrameName:GetText()
	local f
		
	
	GhostPulse3:AddFrame(name)
	f = CreateConfigFrame(name)
	
	if f then
		InterfaceOptions_AddCategory(f)
	end
	
	optionsFrame.FrameName:SetText("")
end)
optionsFrame.CreateFrame:RegisterForClicks("LeftButtonUp")
optionsFrame.CreateFrame:SetPoint("TOPLEFT", optionsFrame, "TOPLEFT", 284, -54)
optionsFrame.CreateFrame:EnableMouse()
optionsFrame.CreateFrame:SetText("Create")
optionsFrame.CreateFrame:SetWidth(112)
optionsFrame.CreateFrame:SetHeight(24)

-- title
optionsFrame.ActiveTitle = optionsFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
optionsFrame.ActiveTitle:SetWidth(340)
optionsFrame.ActiveTitle:SetHeight(20)
optionsFrame.ActiveTitle:SetPoint("TOPLEFT", 12, -88)
optionsFrame.ActiveTitle:SetJustifyH("LEFT")
optionsFrame.ActiveTitle:SetText("Ghost: Pulse is active when...")
	
-- arena check-box
optionsFrame.InArena = CreateFrame("CheckButton", nil, optionsFrame, "OptionsCheckButtonTemplate")
optionsFrame.InArena:SetPoint("TOPLEFT", optionsFrame, "TOPLEFT", 12, -110)
optionsFrame.InArena:SetWidth(24)
optionsFrame.InArena:SetHeight(24)
optionsFrame.InArena:SetChecked(true)
-- arena label
optionsFrame.InArenaLabel = optionsFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
optionsFrame.InArenaLabel:SetWidth(300)
optionsFrame.InArenaLabel:SetHeight(20)
optionsFrame.InArenaLabel:SetPoint("TOPLEFT", 40, -111)
optionsFrame.InArenaLabel:SetJustifyH("LEFT")
optionsFrame.InArenaLabel:SetTextColor(1, 1, 1)
optionsFrame.InArenaLabel:SetText("In Arenas")
optionsFrame.InArena:SetScript("OnClick", function()
	GhostPulse3.currentProfile.activeIn["arena"] = optionsFrame.InArena:GetChecked()
end)
-- BGs check-box
optionsFrame.InBG = CreateFrame("CheckButton", nil, optionsFrame, "OptionsCheckButtonTemplate")
optionsFrame.InBG:SetPoint("TOPLEFT", optionsFrame, "TOPLEFT", 144, -110)
optionsFrame.InBG:SetWidth(24)
optionsFrame.InBG:SetHeight(24)
optionsFrame.InBG:SetChecked(true)
-- BGs label
optionsFrame.InBGLabel = optionsFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
optionsFrame.InBGLabel:SetWidth(300)
optionsFrame.InBGLabel:SetHeight(20)
optionsFrame.InBGLabel:SetPoint("TOPLEFT", 172, -111)
optionsFrame.InBGLabel:SetJustifyH("LEFT")
optionsFrame.InBGLabel:SetTextColor(1, 1, 1)
optionsFrame.InBGLabel:SetText("In Battlegrounds")
optionsFrame.InBG:SetScript("OnClick", function()
	GhostPulse3.currentProfile.activeIn["pvp"] = optionsFrame.InBG:GetChecked()
end)
-- world check-box
optionsFrame.InWorld = CreateFrame("CheckButton", nil, optionsFrame, "OptionsCheckButtonTemplate")
optionsFrame.InWorld:SetPoint("TOPLEFT", optionsFrame, "TOPLEFT", 312, -110)
optionsFrame.InWorld:SetWidth(24)
optionsFrame.InWorld:SetHeight(24)
optionsFrame.InWorld:SetChecked(true)
-- world label
optionsFrame.InWorldLabel = optionsFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
optionsFrame.InWorldLabel:SetWidth(300)
optionsFrame.InWorldLabel:SetHeight(20)
optionsFrame.InWorldLabel:SetPoint("TOPLEFT", 340, -111)
optionsFrame.InWorldLabel:SetJustifyH("LEFT")
optionsFrame.InWorldLabel:SetTextColor(1, 1, 1)
optionsFrame.InWorldLabel:SetText("In World")
optionsFrame.InWorld:SetScript("OnClick", function()
	GhostPulse3.currentProfile.activeIn["none"] = optionsFrame.InWorld:GetChecked()
end)

-- register the frame with WoW
InterfaceOptions_AddCategory(optionsFrame)

-- make the frame available to the slash command handler
GhostPulse3.optionsFrame = optionsFrame

local function OnShow(this)
	if GhostPulse3.locked == 1 or GhostPulse3.locked == true then
		this.Locked:SetChecked(true)
	else
		this.Locked:SetChecked(nil)
	end
	
	this.InArena:SetChecked(GhostPulse3.currentProfile.activeIn["arena"])
	this.InBG:SetChecked(GhostPulse3.currentProfile.activeIn["pvp"])
	this.InWorld:SetChecked(GhostPulse3.currentProfile.activeIn["none"])
end

local function OnEvent(frame, event, whichAddon)
	if event == "ADDON_LOADED" and whichAddon == "GhostPulse3" then
		loadTime = GetTime() + 2
	end
end

local function OnUpdate()
	if loadTime ~= nil and GetTime() >= loadTime then
		-- alerts
		for id, info in pairs(GhostPulse3.categories) do
			local child = CreateAlertConfigFrame(info.name, id)
			
			
			if child then
				InterfaceOptions_AddCategory(child)
			end
		end
		
		-- frames
		for name, info in pairs(GhostPulse3.currentProfile.frames) do
			local child = CreateConfigFrame(name)
			
			
			if child then
				InterfaceOptions_AddCategory(child)
			end
		end

		ShowCurrentSettings()		
		optionsFrame:SetScript("OnUpdate", nil)
	end
end

optionsFrame:RegisterEvent("ADDON_LOADED")
optionsFrame:SetScript("OnEvent", OnEvent)
optionsFrame:SetScript("OnUpdate", OnUpdate)
optionsFrame:SetScript("OnShow", OnShow)

GhostPulse3.AddToFilterList = AddToFilterList
GhostPulse3.RemoveFromFilterList = RemoveFromFilterList
GhostPulse3.ClearFilterList = ClearFilterList
GhostPulse3.ShowFilterList = ShowFilterList
