local spellTextureCache = { }
local spellIdCache = { }
local recordItems = { }
local frm = CreateFrame("Frame")

local startupScanComplete = nil


local function ScanBagItem(bagId, slot)
	local start, duration, enable = GetContainerItemCooldown(bagId, slot)
	

GhostPulse3.debug("Scanning cooldown information for bag "..bagId..", slot "..slot)
	if enable == 1 and start > 0 and duration > 1.5 then
GhostPulse3.debug("Bag "..bagId..", slot "..slot.." has a cooldown")
		local link = GetContainerItemLink(bagId, slot)
		
		
		if link then
GhostPulse3.debug("Bag "..bagId..", slot "..slot.." has a link: "..link)
			local name, _, _, _, _, _, _, _, _, texture = GetItemInfo(link)
			
			
			if name and texture then
GhostPulse3.debug("Bag "..bagId..", slot "..slot.." has a name and a texture: "..name..", "..texture)
				GhostPulse3:AddPulse("bag", name, nil, name.." is ready.", name.." will be ready soon.", texture, 0.25, 1, 0.5, start + duration)
			end
		end
	end
end

local function ScanInventory(slotId, cocall)
	local start, duration, enable = GetInventoryItemCooldown("player", slotId)
	

GhostPulse3.debug("Scanning cooldown information for slot "..slotId)
	if start > 0 and enable == 1 and duration > 1.5 then
GhostPulse3.debug("Slot "..slotId.." has a cooldown")
		local texture = GetInventoryItemTexture("player", slotId)
		
		
		if texture then
GhostPulse3.debug("Found a texture for the item in slot "..slotId)
			local link = GetInventoryItemLink("player", slotId)
			
			
			if link then
GhostPulse3.debug("The item in slot "..slotId.." has a link: "..link)
				local name = GetItemInfo(link)
				

				if name then
GhostPulse3.debug("The item in slot "..slotId.." has a name: "..name)
					GhostPulse3:AddPulse("trinket", name, nil, name.." is ready.", name.." will be ready soon.", texture, 0.25, 1, 0.5, start + duration)
				end
			end
		end
		
		if not cocall then
			if slotId == 13 then
				ScanInventory(14, true)
				
			elseif slotId == 14 then
				ScanInventory(13, true)
			end
		end
	end
end

local function ScanExistingBuffs(auraType)
	local idx = 1
	local whichType
	
	
	if auraType == "BUFF" then
		whichType = "HELPFUL"
	else
		whichType = "HARMFUL"
	end	
	
	while true do
		local name, _, texture, _, buffType, duration, expirationTime = UnitAura("player", idx, whichType)
		
		
		
		if name == nil then
			break
			
		else
			-- remember the texture for next time, so we can
			-- show it when the buff expires or is removed
			if not spellTextureCache[name] then
				spellTextureCache[name] = texture
			end
		end
			
		idx = idx + 1
	end
end

local function AddPulseForBuffGained(buffName, spellId, auraType)
	local idx = 1
	local whichType
	
	
	if auraType == "BUFF" then
		whichType = "HELPFUL"
	else
		whichType = "HARMFUL"
	end	
	
	while true do
		local name, _, texture, _, buffType, duration, expirationTime = UnitAura("player", idx, whichType)
		
		
		
		if name == nil then
			break
			
		else
			if texture then
				-- remember the texture for next time, so we can
				-- show it when the buff expires or is removed
				if not spellTextureCache[name] then
					spellTextureCache[name] = texture
				end
	
				if name == buffName then
					if duration > 1.5 or duration == 0 then
						GhostPulse3:AddPulse(string.lower(auraType).."gain", name, spellId, "You gain "..name..".", "", texture, 0.25, 1, 0.5, GetTime())
						
						if duration > 0 then
							GhostPulse3:AddPulse(string.lower(auraType).."fade", name, spellId, name.." fades.", name.." will fade soon.", texture, 0.25, 1, 0.5, expirationTime)
						end
					end
	
					break
				end
			end
		end
			
		idx = idx + 1
	end
end

local function AddPulseForBuffRemoved(buffName, spellId, auraType)
	if spellTextureCache[buffName] then
		GhostPulse3:RemovePulse(string.lower(auraType).."fade", buffName)
		GhostPulse3:AddPulse(string.lower(auraType).."fade", buffName, spellId, buffName.." fades.", "", spellTextureCache[buffName], 0.25, 1, 0.5, GetTime())
	end
end

local function OnUpdate(frame, elapsed)
	local curTime = GetTime()
	local prune = { }
	

	if not startupScanComplete then
		ScanExistingBuffs("BUFF")
		ScanExistingBuffs("DEBUFF")
		
		startupScanComplete = true
	end
		
	for i = #recordItems, 1, -1 do
		local info = recordItems[i]
		
		
		if info.when ~= nil and info.when <= curTime then
			if info.scanType == "bag" then
				ScanBagItem(info.bagId, info.slot)
				
			elseif info.scanType == "inventory" then
				ScanInventory(info.slotId)
				
			elseif info.scanType == "buff" then
				AddPulseForBuffGained(info.buffName, info.spellId, "BUFF")
				
			elseif info.scanType == "debuff" then
				AddPulseForBuffGained(info.buffName, info.spellId, "DEBUFF")
			end
			
			table.remove(recordItems, i)
		end
	end

	if #recordItems == 0 then	
		frm:SetScript("OnUpdate", nil)
	end
end

hooksecurefunc("UseInventoryItem", function(slotId)
	local info = { }
	
	
GhostPulse3.debug("Detected use of an inventory item in slot "..slotId)

	info.scanType = "inventory"
	info.when = GetTime() + 1.5
	info.slotId = slotId

	table.insert(recordItems, info)	
	frm:SetScript("OnUpdate", OnUpdate)
end)

hooksecurefunc("UseContainerItem", function(bagId, slot, onSelf)
	local info = { }
	
	
GhostPulse3.debug("Detected use of a bag item in bag "..bagId..", slot "..slot)

	info.scanType = "bag"
	info.when = GetTime() + 1.5
	info.bagId = bagId
	info.slot = slot
	
	table.insert(recordItems, info)	
	frm:SetScript("OnUpdate", OnUpdate)
end)

local function HandleSpellAuraGained(sourceGuid, sourceName, sourceFlags, destGuid, destName, destFlags, spellId, spellName, spellSchool, auraType)
	if destGuid == UnitGUID("player") then
		-- scan for the buff
		local info = { }
	
	
GhostPulse3.debug("Detected the gain of a buff or debuff: "..spellName)
	
		info.scanType = string.lower(auraType)
		info.when = GetTime() + 0.5
		info.buffName = spellName
		info.spellId = spellId
			
		table.insert(recordItems, info)	
		frm:SetScript("OnUpdate", OnUpdate)
	end
end

local function HandleSpellAuraRemoved(sourceGuid, sourceName, sourceFlags, destGuid, destName, destFlags, spellId, spellName, spellSchool, auraType)
	if destGuid == UnitGUID("player") then
		-- scan for the (de)buff
		AddPulseForBuffRemoved(spellName, spellId, auraType)
	end
end

local combatEventHandlers = {
	["SPELL_AURA_APPLIED"] = HandleSpellAuraGained,
	["SPELL_AURA_REMOVED"] = HandleSpellAuraRemoved,
}

local function HandleCombatLog(timeStamp, event, ...)
	local f = combatEventHandlers[event]
	
	
	if f then
		f(...)
	end
end

local eventHandlers = {
	["COMBAT_LOG_EVENT_UNFILTERED"] = HandleCombatLog,
}

local function OnEvent(frame, event, ...)
	local f = eventHandlers[event]
	
	
	if f then
		f(...)
	end
end

for name, _ in pairs(eventHandlers) do
	frm:RegisterEvent(name)
end

frm:SetScript("OnEvent", OnEvent)
frm:SetScript("OnUpdate", OnUpdate)

GhostPulse3:RegisterCategory("trinket", "Inventory Cooldowns", true)
GhostPulse3:RegisterCategory("bag", "Bag Item Cooldowns", true)

GhostPulse3:RegisterCategory("buffgain", "Player Buff Gains")
GhostPulse3:RegisterCategory("bufffade", "Player Buff Removal", true, function(alert, name)
	if alert == "earlywarning" then
		GhostPulse3:RemovePulse("bufffade", name)
	end
end)

GhostPulse3:RegisterCategory("debuffgain", "Player Debuff Gains")
GhostPulse3:RegisterCategory("debufffade", "Player Debuff Removal", true, function(alert, name)
	if alert == "earlywarning" then
		GhostPulse3:RemovePulse("debufffade", name)
	end
end)
