local frm = CreateFrame("Frame")
local playerLastAboveThreshold


local classCoords = {
	["WARRIOR"] =		{ 0,		0.25,	0,		0.25 },
	["MAGE"] =			{ 0.25,		0.50,	0,		0.25 },
	["ROGUE"] =			{ 0.50,		0.75,	0,		0.25 },
	["DRUID"] =			{ 0.75,		1.00,	0,		0.25 },

	["HUNTER"] =		{ 0,		0.25,	0.25,	0.50 },
	["SHAMAN"] =		{ 0.25,		0.50,	0.25,	0.50 },
	["PRIEST"] =		{ 0.50,		0.75,	0.25,	0.50 },
	["WARLOCK"] =		{ 0.75,		1.00,	0.25,	0.50 },
	
	["PALADIN"] =		{ 0,		0.25,	0.50,	0.75 },
	["DEATHKNIGHT"] =	{ 0.25,		0.50,	0.50,	0.75 },
}

local function HandleCombatLog(timeStamp, event, sourceGuid, sourceName, sourceFlags, destGuid, destName, destFlags)
	if event == "UNIT_DIED" then
		if UnitExists(destName) and UnitPlayerControlled(destName) then
			local guid = tostring(UnitGUID(destName))
			
			
			if string.sub(guid, 5,5) == "0" then
				local _, class = UnitClass(destName)
				
				
				if class then
					local function ShowClassTexture(texture)
						texture:SetTexture("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes")
						texture:SetTexCoord(classCoords[class][1], classCoords[class][2], classCoords[class][3], classCoords[class][4])
					end
					
					if not UnitIsFeignDeath(destName) then
						GhostPulse3:AddPulse("playerdied", destName, nil, destName.." has died.", "", ShowClassTexture, 0.25, 2, 0.5, GetTime())
					end
				end
			end
		end
	
	elseif event == "SPELL_RESURRECT" then
		if UnitExists(destName) and UnitPlayerControlled(destName) then
			local guid = tostring(UnitGUID(destName))
			
			
			if string.sub(guid, 5,5) == "0" then
				local _, class = UnitClass(destName)
				
				
				if class then
					local function ShowClassTexture(texture)
						texture:SetTexture("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes")
						texture:SetTexCoord(classCoords[class][1], classCoords[class][2], classCoords[class][3], classCoords[class][4])
					end
					
					GhostPulse3:AddPulse("playerressed", destName, nil, destName.." was ressed.", "", ShowClassTexture, 0.25, 2, 0.5, GetTime())
				end
			end
		end
		
	end
end

local function HandleUnitHealth(unitId)
	if unitId == "player" then
		local pc = UnitHealth("player") / UnitHealthMax("player")
		
		
		if pc > 0.35 then
			playerLastAboveThreshold = true
		else
			if playerLastAboveThreshold then
				GhostPulse3:AddPulse("playerhealth", "Low Health", nil, "Low Health", "", "Interface\\Icons\\Spell_Holy_SealOfSacrifice", 0.25, 2, 0.5, GetTime())
			end
			
			playerLastAboveThreshold = false
		end
	end
end

local eventHandlers = {
	["COMBAT_LOG_EVENT_UNFILTERED"] = HandleCombatLog,
	["UNIT_HEALTH"] = HandleUnitHealth,
}

local function OnEvent(frame, event, ...)
	local f = eventHandlers[event]
	
	
	if f then
		f(...)
	end
end

frm:SetScript("OnEvent", OnEvent)

for name, _ in pairs(eventHandlers) do
	frm:RegisterEvent(name)
end

GhostPulse3:RegisterCategory("playerdied", "Player Deaths")
GhostPulse3:RegisterCategory("playerressed", "Player Resurrections")
GhostPulse3:RegisterCategory("playerhealth", "Low Health")
