--[[

	.name
	.texture
	.bookType
	.nextAvailableTime

]]--


local frm = CreateFrame("Frame")
local spellsToRecord = { }
local spellInfo
local rescanSpellbookTime = nil

local function ScanSpellbook(bookType)
	local spellIndex = 1
	

	if spellInfo == nil then
		spellInfo = { }
	end
	
	if bookType == BOOKTYPE_SPELL or ( bookType == BOOKTYPE_PET and select(2, UnitClass("player")) ~= "MAGE" ) then
		while true do
			local name = GetSpellName(spellIndex, bookType)
			

			if name then		
				if not spellInfo[name] then
					local info = { }
					
					
					info.name = name
					info.slotId = spellIndex
					info.texture = select(3, GetSpellInfo(name)) or select(3, GetSpellInfo(name.."()"))
					info.bookType = bookType
					info.nextAvailableTime = nil
					
					spellInfo[name] = info
				end
		
				spellIndex = spellIndex + 1
			else
				break
			end
		end
	else
		while true do
			local name, _, texture = GetPetActionInfo(spellIndex)
			

			if name then
				if not spellInfo[name] then
					local info = { }
					

					info.name = name
					info.slotId = spellIndex
					info.texture = texture
					info.bookType = bookType
					info.nextAvailableTime = nil
					
					spellInfo[name] = info
				end
				
				spellIndex = spellIndex + 1
			else
				break
			end
		end
	end
end

local function UpdateSpellCooldown(name, bookType)
	local rc
	
	
	if name and spellInfo[name] then
		local start, duration, enable


		if bookType == BOOKTYPE_SPELL or ( bookType == BOOKTYPE_PET and select(2, UnitClass("player")) ~= "MAGE" ) then
			start, duration, enable = GetSpellCooldown(spellInfo[name].slotId, bookType)
		else
			start, duration, enable = GetPetActionCooldown(spellInfo[name].slotId)
		end
		
		if enable == 1 and start > 0 then
			local categoryId = "spell"
			
			
			if duration > 1.5 then
				spellInfo[name].nextAvailableTime = start + duration
		
				if bookType == BOOKTYPE_PET then
					categoryId = "petspell"
				end
				
				GhostPulse3:RemovePulse(categoryId, name)
				GhostPulse3:AddPulse(categoryId, name, nil, name.." is ready.", name.." will be ready soon.", spellInfo[name].texture, 0.25, 1, 0.5, spellInfo[name].nextAvailableTime)
			else
				spellInfo[name].nextAvailableTime = nil
			end
			
		elseif enable == 0 then
			if bookType == BOOKTYPE_PET and not UnitExists("playerpet") then
				-- pet's gone!
			else
				rc = true
			end
			
		else
			spellInfo[name].nextAvailableTime = nil
		end
	end
	
	return rc
end

local function UpdateSpellCooldowns()
	for name, info in pairs(spellInfo) do
		UpdateSpellCooldown(name, info.bookType)
	end
end

local function OnUpdate(frame, elapsed)
	local curTime = GetTime()
	local somethingLeft
	
	
	if rescanSpellbookTime ~= nil and GetTime() >= rescanSpellbookTime then
		ScanSpellbook(BOOKTYPE_SPELL)
		ScanSpellbook(BOOKTYPE_PET)
		
		rescanSpellbookTime = nil
	end
	
	for name, info in pairs(spellsToRecord) do
		if info.when ~= nil then
			if info.when <= curTime then
				if not UpdateSpellCooldown(name, info.bookType) then
					info.when = nil
				else
					somethingLeft = true
				end
			else
				somethingLeft = true
			end
		end
	end
	
	if not somethingLeft then
		frame:SetScript("OnUpdate", nil)
	end
end

local function RecordSpell(spellName, bookType, when)
	local info = spellsToRecord[spellName]
	

	if spellInfo == nil then
		ScanSpellbook(BOOKTYPE_SPELL)
		ScanSpellbook(BOOKTYPE_PET)
	end
	
	if info then
		info.bookType = bookType
		info.when = when
	else
		info = { }
		
		info.bookType = bookType
		info.when = when
		
		spellsToRecord[spellName] = info
	end
	
	frm:SetScript("OnUpdate", OnUpdate)
end

local function UpdateSpellbooks()
	spellInfo = nil
	ScanSpellbook(BOOKTYPE_SPELL)
	ScanSpellbook(BOOKTYPE_PET)
	UpdateSpellCooldowns()	
end

local function bit(p)
  return 2 ^ (p - 1)  -- 1-based indexing
end

local function hasbit(x, p)
  return x % (p + p) >= p       
end

local function HandleCombatLog(timeStamp, event, sourceGuid, sourceName, sourceFlags, destGuid, destName, destFlags, spellId, spellName, spellSchool, ...)
	if event == "SPELL_CAST_SUCCESS" or event == "SPELL_CAST_START" then
		local myGuid = UnitGUID("player")
		local petGuid = UnitExists("playerpet") and UnitGUID("playerpet")
		local delayTime = 1.6
		
		
		if sourceGuid == myGuid then
			if event == "SPELL_CAST_START" then
				local castTime = select(7, GetSpellInfo(spellName))
				
				
				if castTime then
					delayTime = castTime / 1000 + 1.0
				end
			end
			
			RecordSpell(spellName, BOOKTYPE_SPELL, GetTime() + delayTime)
		
		elseif sourceGuid == petGuid then
			RecordSpell(spellName, BOOKTYPE_PET, GetTime() + delayTime)
		
		else
			-- enemy spell cast?
			if hasbit(sourceFlags, bit(7)) then
				local categoryId
				
				
				if UnitExists("target") and UnitGUID("target") == sourceGuid then
					categoryId = "targetspellcasts"
				end
				
				if UnitExists("focus") and UnitGUID("focus") == sourceGuid then
					if categoryId == nil then
						categoryId = "focusspellcasts"
					else
						categoryId = "both"
					end
				end
				
				if categoryId then
					local _, _, texture, _, _, _, castTime = GetSpellInfo(spellId)
					local msg
					
					
					if castTime > 0 then
						msg = string.format("%s is casting %s.", sourceName, spellName)
					else
						msg = string.format("%s just cast %s.", sourceName, spellName)
					end
					
					if categoryId ~= "both" then
						GhostPulse3:AddPulse(categoryId, spellName, spellId, msg, "", texture, 0.25, 1, 0.5, GetTime())
					else
						GhostPulse3:AddPulse("targetspellcasts", spellName, spellId, msg, "", texture, 0.25, 1, 0.5, GetTime())
						GhostPulse3:AddPulse("focusspellcasts", spellName, spellId, msg, "", texture, 0.25, 1, 0.5, GetTime())
					end
				end
			end
		end
		
	elseif event == "SPELL_SUMMON" then
		rescanSpellbookTime = GetTime() + 1.6
		frm:SetScript("OnUpdate", OnUpdate)
	end
end

local eventHandlers = {
	["COMBAT_LOG_EVENT_UNFILTERED"] = HandleCombatLog,

	["LEARNED_SPELL_IN_TAB"] = function()
		UpdateSpellbooks()
	end,
}

local function OnEvent(frame, event, ...)
	local f = eventHandlers[event]
	
	
	if f then
		f(...)
	end
end

frm:SetScript("OnEvent", OnEvent)

for name, _ in pairs(eventHandlers) do
	frm:RegisterEvent(name)
end

GhostPulse3:RegisterCategory("targetspellcasts", "Target Spell Casts", false)
GhostPulse3:RegisterCategory("focusspellcasts", "Focus Spell Casts", false)
GhostPulse3:RegisterCategory("spell", "Player Spell Cooldowns", true)
GhostPulse3:RegisterCategory("petspell", "Pet Spell Cooldowns", true)

