Inquest = LibStub("AceAddon-3.0"):NewAddon("Inquest", "AceHook-3.0", "AceTimer-3.0", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("Inquest", true)
local InquestDataObject

local string_format = string.format

--Create default settings
local defaults = {
	profile = {
		showQuestLevels = true, 
		showQuestTags = true, 
		autoCompleteQuests = false, 
		showLdbLabel = true, 
		
		questLogLocked = true, 
		questCompleteMessage = false, 
		echoObjectiveProgressToParty = false,
		echoQuestCompleteToParty = true, 
		
		questCompleteSound = true, 
		questCompleteSoundFile = "Work Complete",
		objectiveCompleteSound = false, 
		objectiveCompleteSoundFile = "Cow", 
		objectiveProgressSound = false, 
		objectiveProgressSoundFile = "More Work?",
		
		trackerLocked = WatchFrame.locked,
		trackerInOriginalPosition = true, 
		hideTrackerBackdrop = false, 
		trackerBackdropAlpha = 0.0, 
		autoQuestWatch = false, 
		autoQuestProgress = true, 
		showItemButtons = true, 
		leftAlignItemButtons = false, 
		removeCompletedObjectives = false, 
		removeCompletedQuests = false, 
		autoResizeTracker = true,
		trackerHorizontalGrowth = "1-left", 
		trackerVerticalGrowth = "2-down", 
		trackerCollapsePoint = "TOP-RIGHT", 
		trackerFont = "Friz Quadrata TT", 
		trackerFontSize = 12, 
		trackerLineSpacing = 10, 
		hideTrackerWhen = {
			always = false, 
			empty = false, 
			inDungeon = false, 
			inRaid = true, 
			inBattleground = true,
			inArena = true, 
		},
		clickBindings = {
			left = "2-OnClick_Open", 
			right = "7-OnClick_ShowBlizMenu", 
			["*"] = "1-none"
		},
		questItemTooltip = true, 
		trackerTooltip = true, 
	},
	char = {
		numQuestsWatched = 0,
		numAchievementsWatched = 0, 
		watchedQuests = {}, 
		watchTimers = {}, 
		watchedAchievements = {}, 
	}
}

function Inquest:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("InquestDB", defaults)
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")

	self:RegisterSounds()
	self:SetupOptions()
	
	InquestDataObject = LibStub("LibDataBroker-1.1"):NewDataObject("Inquest", {
		type = "data source",
		text = "0/0",
		icon = "Interface\\QuestFrame\\UI-QuestLog-BookIcon", 
		OnClick = function(frame, button) 
			if(button == "RightButton") then 
				self:ShowOptions() 
			else 
				self:SetTrackerVisibility(not WatchFrame:IsShown()) 
			end
		end, 
		OnTooltipShow = function(tt) 
			tt:AddLine("Inquest")
			tt:AddLine("Click to toggle the tracker")
			tt:AddLine("Right-click to open the options menu") 
		end,
	})
end

function Inquest:OnEnable()
	self:InitializeQuestLog()
	self:InitializeTracker()
	self:InitializeTooltip()
end

function Inquest:OnDisable()
	LibStub("LibQuixote-2.0").UnregisterAll(self)
	self:UnhookAll()
	self:UnregisterEvent("QUEST_PROGRESS")
	self:UnregisterEvent("QUEST_COMPLETE")
	self:UnregisterEvent("QUEST_ACCEPTED")
	self:UnregisterEvent("QUEST_WATCH_UPDATE")
	self:UnregisterEvent("ACHIEVEMENT_EARNED")
end

function Inquest:OnProfileChanged(event, database, newProfileKey)
	--Quest log stuff
	self:SetQuestLogPosition()
	self:SetQuestLogLocked()
	
	--Tracker stuff
	self:LoadTrackerSettings()
	
	QuestLog_Update()
	WatchFrame_Update()
end

function Inquest:UpdateLdbText()
	local _, numQuests = GetNumQuestLogEntries()
	InquestDataObject.text = string_format("%d/%d", numQuests, MAX_QUESTLOG_QUESTS)
	if(self.db.profile.showLdbLabel) then InquestDataObject.text = L["Quests"] .. ": " .. InquestDataObject.text end
end

function Inquest:RegisterSounds()
	local LSM = LibStub("LibSharedMedia-3.0")
	LSM:Register("sound", "Work Complete", [[Sound\Creature\Peon\PeonBuildingComplete1.wav]])
	LSM:Register("sound", "Exploration", [[Sound\Interface\GnomeExploration.wav]])
	LSM:Register("sound", "Flag Captured 1", [[Sound\Interface\PVPFlagCapturedmono.wav]])
	LSM:Register("sound", "Flag Captured 2", [[Sound\Interface\PVPFlagCapturedHordemono.wav]])
	LSM:Register("sound", "Raid Warning", [[Sound\Interface\RaidWarning.wav]])
	LSM:Register("sound", "Ready Check", [[Sound\Interface\ReadyCheck.wav]])
	LSM:Register("sound", "Level Up 1", [[Sound\Interface\LevelUp.wav]])
	LSM:Register("sound", "Level Up 2", [[Sound\Interface\levelup2.wav]])
	LSM:Register("sound", "Cow", [[Sound\Creature\Cow\CowDeath.wav]])
	LSM:Register("sound", "Final Fantasy Victory", [[Interface\Addons\Inquest\Sounds\Victory.mp3]])
	LSM:Register("sound", "More Work?", [[Sound\Creature\Peasant\PeasantWhat3.wav]])
end

--Returns a quest log title with tags added
local tags = {Elite = "+", Group = "G", Dungeon = "D", Raid = "R", PvP = "P", Heroic = "H"}
function Inquest:GetQuestLogTaggedTitle(questIndex)
	local originalTitle, level, tagName, groupSize, isHeader, _, isComplete, isDaily = GetQuestLogTitle(questIndex)
	if(isHeader) then return nil end
	local tag = ""
	if(self.db.profile.showQuestTags) then
		if(tags[tagName]) then tag = tags[tagName] end
		if(isDaily) then tag = "Y" .. tag end
		if(tagName == "Group" and groupSize > 0) then tag = tag .. groupSize end
	end
	if(self.db.profile.showQuestLevels) then tag = level .. tag end
	
	if (tag == "") then return originalTitle end
	return "[" .. tag .. "] " .. originalTitle, isComplete
end
