local L = LibStub("AceLocale-3.0"):GetLocale("Inquest", true)

local options
local function GenerateOptions()
	if(not options) then
		local clickBindingOptions = {
			["1-none"] = L["None"], 
			["2-OnClick_Open"] = L["Open target item"], 
			["3-OnClick_StopTracking"] = L["Stop tracking target item"], 
			["4-OnClick_ShareQuest"] = L["Share target quest"], 
			["5-OnClick_AbandonQuest"] = L["Abandon target quest"], 
			["6-OnClick_InsertChatLink"] = L["Insert chat link"], 
			["7-OnClick_ShowBlizMenu"] = L["Show Blizzard menu"],
		}
		
		options = {
			type = "group",
			get = function(info) return Inquest.db.profile[info[#info]] end, 
			set = function(info, val) Inquest.db.profile[info[#info]] = val end, 
			args = {
				general = {
					name = L["General Options"], 
					type = "group", 
					guiInline = true, 
					order = 1, 
					args = {
						showQuestLevels = {
							name = L["Show quest levels"], 
							desc = L["Show quest levels in the quest log, tracker, and tooltips"], 
							type = "toggle", 
							order = 1, 
							set = function(info, val) 
								Inquest.db.profile.showQuestLevels = val 
								WatchFrame_Update()
							end
						}, 
						showQuestTags = {
							name = L["Show quest tags"], 
							desc = L["Show quest tags in the quest log, tracker, and tooltips"], 
							type = "toggle", 
							order = 2, 
							set = function(info, val) 
								Inquest.db.profile.showQuestTags = val 
								WatchFrame_Update()
							end
						},
						autoCompleteQuests = {
							name = L["Auto-complete quests with no reward choices"], 
							desc = L["Automatically complete quests that don't have a reward choice"], 
							type = "toggle", 
							width = "full", 
							order = 5, 
						},
						showLdbLabel = {
							name = L["Show Quests: in LDB feed text"], 
							desc = L["Show a label before the numbers in the LDB feed's text"], 
							type = "toggle", 
							width = "full", 
							order = 7, 
							set = function(info, val)
								Inquest.db.profile.showLdbLabel = val
								Inquest:UpdateLdbText()
							end
						},
					}
				},
				questLog = {
					name = L["Quest Log Options"], 
					type = "group", 
					guiInline = true, 
					order = 3, 
					args = {
						questLogLocked = {
							name = L["Lock quest log"], 
							desc = L["Lock the quest log's position"], 
							type = "toggle",  
							order = 0, 
							set = function(info, val) 
								Inquest.db.profile.questLogLocked = val 
								Inquest:SetQuestLogLocked(val)
							end
						}, 
						resetLogPosition = {
							name = L["Reset log position"], 
							desc = L["Reset the quest log's position"], 
							type = "execute",
							order = 2, 
							func = function()
								Inquest.db.profile.logxOffset = nil
								Inquest.db.profile.logyOffset = nil
								Inquest:SetQuestLogPosition()
							end
						}, 
						questCompleteMessage = {
							name = L["Show a message when you complete a quest"], 
							desc = L["Show a message when you complete a quest"], 
							type = "toggle", 
							width = "full",
							order = 4, 
						}, 
						echoObjectiveProgressToParty = {
							name = L["Echo quest progress to party members"], 
							desc = L["Show a message in party chat when you make progress on a quest objective"], 
							type = "toggle", 
							width = "full", 
							order = 6, 
						},
						echoQuestCompleteToParty = {
							name = L["Echo quest and objective completion to party members"], 
							desc = L["Show a message in party chat when you complete a quest or quest objective"], 
							type = "toggle", 
							width = "full", 
							order = 8, 
						},
						sounds = {
							name = L["Sound Options"], 
							type = "group", 
							guiInline = true, 
							order = 10, 
							args = {
								questCompleteSound = {
									name = L["Play a sound when you complete a quest"], 
									desc = L["Play a sound when you complete a quest"], 
									type = "toggle",
									width = "full",
									order = 0, 
								}, 
								questCompleteSoundFile = {
									name = L["Quest completion sound"], 
									desc = L["Choose the sound to play when you complete a quest"], 
									type = "select", 
									dialogControl = "LSM30_Sound", 
									order = 2, 
									values = AceGUIWidgetLSMlists.sound, 
									disabled = function() return not Inquest.db.profile.questCompleteSound end,
								},
								objectiveCompleteSound = {
									name = L["Play a sound when you complete a quest objective"], 
									desc = L["Play a sound when you complete a quest objective"], 
									type = "toggle",
									width = "full",
									order = 4, 
								}, 
								objectiveCompleteSoundFile = {
									name = L["Objective completion sound"], 
									desc = L["Choose the sound to play when you complete a quest objective"], 
									type = "select", 
									dialogControl = "LSM30_Sound", 
									order = 6, 
									values = AceGUIWidgetLSMlists.sound, 
									disabled = function() return not Inquest.db.profile.objectiveCompleteSound end,
								},
								objectiveProgressSound = {
									name = L["Play a sound when you make progress on a quest objective"], 
									desc = L["Play a sound when you make progress on a quest objective"], 
									type = "toggle",
									width = "full",
									order = 8, 
								}, 
								objectiveProgressSoundFile = {
									name = L["Objective progress sound"], 
									desc = L["Choose the sound to play when you progress a quest objective"], 
									type = "select", 
									dialogControl = "LSM30_Sound", 
									order = 10, 
									values = AceGUIWidgetLSMlists.sound, 
									disabled = function() return not Inquest.db.profile.objectiveProgressSound end,
								},
							}
						}
					}
				},
				tracker = {
					name = L["Tracker Options"], 
					type = "group", 
					guiInline = true,
					order = 4, 
					args = {
						trackerLocked = {
							name = L["Lock tracker"], 
							desc = L["Lock the tracker's size and position (if resizing manually)"], 
							type = "toggle", 
							order = 0, 
							set = WatchFrame_ToggleLock
						},
						resetTrackerPosition = {
							name = L["Reset tracker position"],
							desc = L["Reset the tracker's position"], 
							type = "execute", 
							order = 2, 
							func = function() 
								Inquest.db.profile.trackerInOriginalPosition = true
								Inquest:SetTrackerPosition()
							end
						},
						showItemButtons = {
							name = L["Show quest item buttons on the tracker"], 
							desc = L["Show buttons on the tracker for quests with special items"], 
							type = "toggle", 
							width = "full", 
							order = 3, 
							set = function(info, val)
								Inquest.db.profile.showItemButtons = val
								WatchFrame_Update()
							end
						},
						leftAlignItemButtons = {
							name = L["Left align quest item buttons on the tracker"], 
							desc = L["Left align quest item buttons on the tracker"], 
							type = "toggle", 
							width = "full", 
							order = 4, 
							disabled = function() return not Inquest.db.profile.showItemButtons end, 
							set = function(info, val)
								Inquest.db.profile.leftAlignItemButtons = val
								WatchFrame_Update()
							end
						},
						autoQuestWatch = {
							name = L["Automatic quest tracking"], 
							desc = L["Automatically track quests as soon as you obtain them"], 
							type = "toggle", 
							width = "full", 
							order = 5, 
							set = function(info, val) 
								Inquest.db.profile.autoQuestWatch = val
								InterfaceOptionsObjectivesPanelAutoQuestTracking:Click("LeftButton")
							end
						}, 
						autoQuestProgress = {
							name = L["Automatic quest progress updates"], 
							desc = L["Automatically track quests for 5 minutes when you make progress on a quest objective"], 
							type = "toggle", 
							width = "full", 
							order = 6, 
							set = function(info, val)
								Inquest.db.profile.autoQuestProgress = val
								InterfaceOptionsObjectivesPanelAutoQuestProgress:Click("LeftButton")
							end
						},
						removeCompletedObjectives = {
							name = L["Remove completed quest objectives from the tracker"], 
							desc = L["Remove completed quest objectives from the tracker"], 
							type = "toggle", 
							width = "full", 
							order = 8,  
							set = function(info, val)
								Inquest.db.profile.removeCompletedObjectives = val
								WatchFrame_Update()
							end
						}, 
						removeCompletedQuests = {
							name = L["Remove completed quests from the tracker"], 
							desc = L["Remove completed quests from the tracker"], 
							type = "toggle",
							width = "full", 
							order = 10, 
							set = function(info, val) 
								Inquest.db.profile.removeCompletedQuests = val
								if(val) then Inquest:RemoveCompletedQuestWatches() end
							end
						},
						trackerBackdropGroup = {
							name = L["Backdrop Options"], 
							type = "group",
							guiInline = true, 
							order = 12, 
							args = {
								hideTrackerBackdrop = {
									name = L["Permanently hide tracker backdrop"], 
									desc = L["Prevent the tracker backdrop from showing on mouseover"], 
									type = "toggle", 
									width = "full", 
									order = 12, 
									set = function(info, val)
										Inquest.db.profile.hideTrackerBackdrop = val
										Inquest:SetTrackerBackdropHidden(val)
									end
								},
								trackerBackdropAlpha = {
									name = L["Tracker backdrop opacity"], 
									desc = L["Tracker backdrop opacity"], 
									type = "range", 
									isPercent = true, 
									min = 0, 
									max = 1, 
									step = 0.01, 
									order = 14, 
									disabled = function() return Inquest.db.profile.hideTrackerBackdrop end,
									set = function(info, val) WatchFrame_SetBaseAlpha(1 - val) end
								},
							}
						},
						sizeGroup = {
							name = L["Size Options"], 
							type = "group", 
							guiInline = true,
							order = 13, 
							args = {
								autoResizeTracker = {
									name = L["Automatically resize the tracker"], 
									desc = L["Automatically resize the tracker to fit its contents"], 
									type = "toggle",
									width = "full", 
									order = 18, 
									set = function(info, val) 
										Inquest.db.profile.autoResizeTracker = val 
										WatchFrame_Update()
									end
								}, 
								trackerHorizontalGrowth = {
									name = L["Horizontal growth direction"], 
									desc = L["Direction in which the tracker grows horizontally"], 
									type = "select", 
									values = {
										["1-left"] = L["Grow Left"], 
										["2-right"] = L["Grow Right"],
										["3-none"] = L["None"]
									},
									order = 20, 
									disabled = function() return not Inquest.db.profile.autoResizeTracker end,
									set = function(info, val) 
										local oldVal = Inquest.db.profile.trackerHorizontalGrowth
										Inquest.db.profile.trackerHorizontalGrowth = val 
										if(oldVal == "3-none" and val ~= "3-none") then WatchFrame_Update() end
									end
								}, 
								trackerVerticalGrowth = {
									name = L["Vertical growth direction"], 
									desc = L["Direction in which the tracker grows vertically"], 
									type = "select", 
									values = {
										["1-up"] = L["Grow Up"], 
										["2-down"] = L["Grow Down"]
									},  
									order = 22, 
									disabled = function() return not Inquest.db.profile.autoResizeTracker end, 
									set = function(info, val) 
										Inquest.db.profile.trackerVerticalGrowth = val 
									end
								},
								trackerCollapsePoint = {
									name = L["Collapse tracker to ..."], 
									desc = L["Corner to which the tracker collapses when minimized"], 
									type = "select", 
									values = {
										["TOP-RIGHT"] = L["Top Right"], 
										["TOP-LEFT"] = L["Top Left"], 
										["BOTTOM-RIGHT"] = L["Bottom Right"], 
										["BOTTOM-LEFT"] = L["Bottom Left"], 
									},
									order = 24, 
								},
							}
						},
						fontGroup = {
							name = L["Font and Spacing Options"], 
							type = "group", 
							guiInline = true, 
							order = 14, 
							args = {
								trackerFont = {
									name = L["Tracker font"], 
									desc = L["Tracker font"], 
									type = "select", 
									dialogControl = "LSM30_Font", 
									values = AceGUIWidgetLSMlists.font, 
									order = 24, 
									set = function(info, val) 
										Inquest.db.profile.trackerFont = val 
										WatchFrame_Update()
									end
								}, 
								trackerFontSize = {
									name = L["Tracker font size"], 
									desc = L["Tracker font size"], 
									type = "range", 
									min = 8, 
									max = 24, 
									step = 1, 
									order = 26,  
									set = function(info, val) 
										Inquest.db.profile.trackerFontSize = val
										Inquest:UpdateTrackerFontSpacing(val) 
										WatchFrame_Update()
									end
								},
								trackerLineSpacing = {
									name = L["Tracker line spacing"], 
									desc = L["Line spacing between quests or achievements on the tracker"], 
									type = "range", 
									min = 0, 
									max = 10, 
									step = 1, 
									order = 27, 
									set = function(info, val)
										Inquest.db.profile.trackerLineSpacing = val
										WATCHFRAME_QUEST_OFFSET = val
										WatchFrame_Update()
									end
								},
							}
						},
						hideTrackerWhen = {
							name = L["Hide the Tracker When ..."], 
							type = "group", 
							guiInline = true,
							order = 16, 
							get = function(info) return Inquest.db.profile.hideTrackerWhen[info[#info]] end,
							set = function(info, val) 
								Inquest.db.profile.hideTrackerWhen[info[#info]] = val 
								Inquest:AutoHideTracker()
							end,
							args = {
								always = {
									name = L["Always"], 
									desc = L["Permanently hide the tracker"], 
									type = "toggle", 
									order = 0, 
								},
								empty = {
									name = L["Empty"], 
									desc = L["Hide the tracker when it is empty"], 
									type = "toggle", 
									order = 2, 
								},
								inDungeon = {
									name = L["You are in a 5 man dungeon"], 
									desc = L["Hide the tracker when you are in a 5 man dungeon"], 
									type = "toggle", 
									width = "full", 
									order = 4, 
								}, 
								inRaid = {
									name = L["You are in a raid instance"], 
									desc = L["Hide the tracker when you are in a raid instance"], 
									type = "toggle", 
									width = "full", 
									order = 6, 
								},
								inBattleground = {
									name = L["You are in a battleground"], 
									desc = L["Hide the tracker when you are in a battleground"], 
									type = "toggle", 
									width = "full", 
									order = 8, 
								},
								inArena = {
									name = L["You are in an arena match"], 
									desc = L["Hide the tracker when you are in an arena match"], 
									type = "toggle", 
									width = "full", 
									order = 10, 
								},
							}
						},
						clickBindings = {
							name = L["Click Binding Options"], 
							type = "group", 
							guiInline = true,
							get = function(info) return Inquest.db.profile.clickBindings[info[#info]] end,
							set = function(info, val) Inquest.db.profile.clickBindings[info[#info]] = val end,
							order = 28,
							args = {
								left = {
									name = L["Left Click"], 
									desc = L["Select a command to bind to left click"], 
									type = "select", 
									values = clickBindingOptions, 
									order = 1,
								},
								right = {
									name = L["Right Click"], 
									desc = L["Select a command to bind to right click"], 
									type = "select", 
									values = clickBindingOptions, 
									order = 2,
								},
								shiftLeft = {
									name = L["Shift + Left Click"], 
									desc = L["Select a command to bind to shift + left click"], 
									type = "select", 
									values = clickBindingOptions, 
									order = 3,
								},
								shiftRight = {
									name = L["Shift + Right Click"], 
									desc = L["Select a command to bind to shift + right click"], 
									type = "select", 
									values = clickBindingOptions, 
									order = 4,
								},
								ctrlLeft = {
									name = L["Ctrl + Left Click"], 
									desc = L["Select a command to bind to control + left click"], 
									type = "select", 
									values = clickBindingOptions, 
									order = 5,
								},
								ctrlRight = {
									name = L["Control + Right Click"], 
									desc = L["Select a command to bind to control + right click"], 
									type = "select", 
									values = clickBindingOptions, 
									order = 6,
								},
								altLeft = {
									name = L["Alt + Left Click"], 
									desc = L["Select a command to bind to alt + left click"], 
									type = "select", 
									values = clickBindingOptions, 
									order = 7,
								},
								altRight = {
									name = L["Alt + Right Click"], 
									desc = L["Select a command to bind to alt + right click"], 
									type = "select", 
									values = clickBindingOptions, 
									order = 8
								},
							}
						},
					}
				}, 
				tooltip = {
					name = L["Tooltip Options"], 
					type = "group", 
					guiInline = true, 
					order = 5, 
					args = {
						questItemTooltip = {
							name = L["Show quest information on item tooltips"], 
							desc = L["Show quest information on item tooltips"], 
							type = "toggle", 
							width = "full", 
							order = 2, 
						}, 
						trackerTooltip = {
							name = L["Show quest and achievement tooltips on the tracker"], 
							desc = L["Show quest and achievement tooltips on the tracker"], 
							type = "toggle", 
							width = "full", 
							order = 3, 
						},
					}
				},
			}
		}
		
		options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(Inquest.db)
		options.args.profiles.order = -1
	end
	
	return options
end
	
function Inquest:SetupOptions()
	self.optionsFrames = {}

	LibStub("AceConfig-3.0"):RegisterOptionsTable("Inquest", GenerateOptions)
	
	local ACD = LibStub("AceConfigDialog-3.0")
	self.optionsFrames.general = ACD:AddToBlizOptions("Inquest", nil, nil, "general")
	self.optionsFrames.questLog = ACD:AddToBlizOptions("Inquest", L["Quest Log Options"], "Inquest", "questLog")
	self.optionsFrames.tracker = ACD:AddToBlizOptions("Inquest", L["Tracker Options"], "Inquest", "tracker")
	self.optionsFrames.tooltip = ACD:AddToBlizOptions("Inquest", L["Tooltip Options"], "Inquest", "tooltip")
	self.optionsFrames.profiles = ACD:AddToBlizOptions("Inquest", L["Profiles"], "Inquest", "profiles")
	
	self:RegisterChatCommand("Inquest", "ShowOptions")
end

function Inquest:ShowOptions()
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrames.profiles)
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrames.general)
end
