local quixote = LibStub("LibQuixote-2.0")
local LSM = LibStub("LibSharedMedia-3.0")

function Inquest:InitializeQuestLog()
	--Make quest log frame draggable
	QuestLogFrame:EnableMouse(true)
	QuestLogFrame:SetMovable(true)
	QuestLogFrame:SetClampedToScreen(true)
	self:HookScript(QuestLogFrame, "OnDragStart", "QuestLog_OnDragStart")
	self:HookScript(QuestLogFrame, "OnDragStop", "QuestLog_OnDragStop")
	
	--Set locked state of the quest log frame
	self:SetQuestLogLocked(self.db.profile.questLogLocked)
	
	self:SecureHook("QuestLogTitleButton_OnClick")
	self:SecureHook("QuestLog_Update")
	--Have to set the scroll frame's update function again because of the hook
	QuestLogScrollFrame.update = QuestLog_Update
	
	--Restore pre-3.2 quest log open functionality
	self:SecureHook("QuestLog_OpenToQuest")
	
	--Reposition quest log when it is shown
	self:HookScript(QuestLogFrame, "OnShow", function() self:SetQuestLogPosition() end)
	
	quixote.RegisterCallback(self, "Objective_Update", "Objective_Update")
	
	self:RegisterEvent("QUEST_PROGRESS")
	self:RegisterEvent("QUEST_COMPLETE")
end

function Inquest:Objective_Update(event, questTitle, uid, desc, had, got, needed)
	local questComplete, _
	uid, _, _, _, _, _, questComplete = quixote:GetQuestByUid(uid)
	local inParty = GetNumPartyMembers() > 0 and GetNumRaidMembers() == 0
	if(uid) then
		if(got >= needed) then
			if(not questComplete and self.db.profile.objectiveCompleteSound) then
				PlaySoundFile(LSM:Fetch("sound", self.db.profile.objectiveCompleteSoundFile))
			end
			if(self.db.profile.echoQuestCompleteToParty and inParty) then
				SendChatMessage(desc .. ": " .. got .. "/" .. needed, "PARTY")
			end
		elseif(got > had) then
			if(self.db.profile.objectiveProgressSound) then
				PlaySoundFile(LSM:Fetch("sound", self.db.profile.objectiveProgressSoundFile))
			end
			if(self.db.profile.echoObjectiveProgressToParty and inParty) then
				SendChatMessage(desc .. ": " .. got .. "/" .. needed, "PARTY")
			end
		end
	end
end

function Inquest:QUEST_PROGRESS()
	if(self.db.profile.autoCompleteQuests) then
		CompleteQuest()
	end
end

function Inquest:QUEST_COMPLETE()
	if(self.db.profile.autoCompleteQuests and GetNumQuestChoices() == 0) then
		GetQuestReward(QuestFrameRewardPanel.itemChoice)
	end
end

function Inquest:CanWatchQuest(questIndex)
	if((not WatchFrame:IsUserPlaced()) and ArenaEnemyFrames and ArenaEnemyFrames:IsShown()) then
		UIErrorsFrame:AddMessage(OBJECTIVES_WATCH_QUESTS_ARENA, 1.0, 0.1, 0.1, 1.0)
		return false
	elseif(WatchFrame_GetRemainingSpace() < WatchFrame_GetHeightNeededForQuest(questIndex)) then
		UIErrorsFrame:AddMessage(OBJECTIVES_WATCH_TOO_MANY, 1.0, 0.1, 0.1, 1.0)
		return false
	end
	
	return true
end

function Inquest:QuestLogTitleButton_OnClick(titleButton, mouseButton)
	local questName = titleButton:GetText()
	local questIndex = titleButton:GetID()
	if(IsModifiedClick("QUESTWATCHTOGGLE") and titleButton.isHeader) then
		local buttons = QuestLogScrollFrame.buttons
		local headerSortIndex = GetQuestSortIndex(questIndex)
		local currentQuestIndex = questIndex + 1
		local allQuestsInHeaderWatched = true
		--If some quests in the zone are not tracked, track them
		while(GetQuestSortIndex(currentQuestIndex) == headerSortIndex) do
			if(not IsQuestWatched(currentQuestIndex)) then
				allQuestsInHeaderWatched = false
				if(self:CanWatchQuest(currentQuestIndex)) then
					AddQuestWatch(currentQuestIndex)
				end
			end
			currentQuestIndex = currentQuestIndex + 1
		end
		
		--Otherwise untrack all quests in the zone
		if(allQuestsInHeaderWatched) then
			for qi=questIndex + 1, currentQuestIndex - 1 do
				RemoveQuestWatch(qi)
			end
		end
		
		WatchFrame_Update()
		QuestLog_Update()
	end
end

function Inquest:QuestLog_OpenToQuest()
	if(QuestLogDetailFrame:IsShown()) then
		ShowUIPanel(QuestLogFrame)
	else
		HideUIPanel(QuestLogFrame)
	end
end

function Inquest:QuestLog_Update()
	self:UpdateLdbText()
	
	local numEntries = GetNumQuestLogEntries()
	local scrollOffset = HybridScrollFrame_GetOffset(QuestLogScrollFrame)
	local buttons = QuestLogScrollFrame.buttons
	local numButtons = #buttons
	
	for i=1, numButtons do
		local questIndex = i + scrollOffset
		local questLogTitle = buttons[i]
		
		--Set the new, tagged title for the quest
		if(questIndex <= numEntries) then
			local title = self:GetQuestLogTaggedTitle(questIndex)
			if(title) then
				questLogTitle:SetText(" " .. title)
			end
		end
		
		QuestLogTitleButton_Resize(questLogTitle)
	end
end

function Inquest:SetQuestLogPosition()
	if(QuestLogFrame.moving) then return end
	
	QuestLogFrame:ClearAllPoints()
	if(self.db.profile.logxOffset and self.db.profile.logyOffset) then
		QuestLogFrame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", self.db.profile.logxOffset, self.db.profile.logyOffset)
	else
		QuestLogFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 0, -104)
	end
end

function Inquest:SetQuestLogLocked(questLogLocked)
	questLogLocked = questLogLocked or self.db.profile.questLogLocked
	if(questLogLocked) then
		QuestLogFrame:RegisterForDrag(nil)
	else
		QuestLogFrame:RegisterForDrag("LeftButton")
	end
end

function Inquest:SaveQuestLogPosition()
	self.db.profile.logxOffset = QuestLogFrame:GetLeft()
	self.db.profile.logyOffset = QuestLogFrame:GetTop()
end

function Inquest:QuestLog_OnDragStart()
	QuestLogFrame.moving = true
	QuestLogFrame:StartMoving()
end

function Inquest:QuestLog_OnDragStop()
	QuestLogFrame.moving = false
	QuestLogFrame:StopMovingOrSizing()
	self:SaveQuestLogPosition()
end
